"""OCI Events Service Module."""

from typing import List

import oci
from pydantic import BaseModel

from prowler.lib.logger import logger
from prowler.providers.oraclecloud.lib.service.service import OCIService


class Events(OCIService):
    """OCI Events Service class to retrieve event rules and notification topics."""

    def __init__(self, provider):
        """Initialize the Events service."""
        super().__init__("events", provider)
        self.rules = []
        self.topics = []
        self.__threading_call__(self.__list_rules__)
        self.__threading_call__(self.__list_topics__)

    def __get_client__(self, region):
        """Get the Events client for a region."""
        return self._create_oci_client(
            oci.events.EventsClient, config_overrides={"region": region}
        )

    def __list_rules__(self, regional_client):
        """List all event rules."""
        try:
            # Create events client for this region
            events_client = self.__get_client__(regional_client.region)
            if not events_client:
                return

            logger.info(f"Events - Listing Rules in {regional_client.region}...")

            for compartment in self.audited_compartments:
                try:
                    logger.info(
                        f"Events - Checking compartment {compartment.name} ({compartment.id})..."
                    )
                    rules = oci.pagination.list_call_get_all_results(
                        events_client.list_rules, compartment_id=compartment.id
                    ).data

                    logger.info(
                        f"Events - Found {len(rules)} rules in compartment {compartment.name}"
                    )

                    for rule in rules:
                        if rule.lifecycle_state != "DELETED":
                            # Get full rule details including actions
                            try:
                                full_rule = events_client.get_rule(rule_id=rule.id).data

                                # Extract actions from the full rule details
                                actions_list = []
                                if hasattr(full_rule, "actions") and full_rule.actions:
                                    if hasattr(full_rule.actions, "actions"):
                                        # Convert action objects to dictionaries for JSON serialization
                                        for action in full_rule.actions.actions:
                                            action_dict = {
                                                "action_type": (
                                                    action.action_type
                                                    if hasattr(action, "action_type")
                                                    else None
                                                ),
                                                "is_enabled": (
                                                    action.is_enabled
                                                    if hasattr(action, "is_enabled")
                                                    else False
                                                ),
                                                "id": (
                                                    action.id
                                                    if hasattr(action, "id")
                                                    else None
                                                ),
                                            }
                                            actions_list.append(action_dict)

                                self.rules.append(
                                    Rule(
                                        id=full_rule.id,
                                        name=(
                                            full_rule.display_name
                                            if hasattr(full_rule, "display_name")
                                            else full_rule.id
                                        ),
                                        compartment_id=compartment.id,
                                        region=regional_client.region,
                                        lifecycle_state=full_rule.lifecycle_state,
                                        condition=(
                                            full_rule.condition
                                            if hasattr(full_rule, "condition")
                                            else ""
                                        ),
                                        is_enabled=(
                                            full_rule.is_enabled
                                            if hasattr(full_rule, "is_enabled")
                                            else False
                                        ),
                                        actions=actions_list,
                                    )
                                )
                            except Exception as error:
                                logger.error(
                                    f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                                )
                except Exception as error:
                    logger.error(
                        f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                    )
                    continue
        except Exception as error:
            logger.error(
                f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )

    def __list_topics__(self, regional_client):
        """List all notification topics."""
        try:
            # Control plane client for listing topics
            ons_control_client = self._create_oci_client(
                oci.ons.NotificationControlPlaneClient,
                config_overrides={"region": regional_client.region},
            )

            # Data plane client for listing subscriptions
            ons_data_client = self._create_oci_client(
                oci.ons.NotificationDataPlaneClient,
                config_overrides={"region": regional_client.region},
            )

            logger.info(f"Events - Listing Topics in {regional_client.region}...")

            # First, get all subscriptions in this compartment for later matching
            all_subscriptions = {}
            for compartment in self.audited_compartments:
                try:
                    subs = oci.pagination.list_call_get_all_results(
                        ons_data_client.list_subscriptions,
                        compartment_id=compartment.id,
                    ).data

                    # Group subscriptions by topic_id
                    for sub in subs:
                        topic_id = sub.topic_id
                        if topic_id not in all_subscriptions:
                            all_subscriptions[topic_id] = []
                        if sub.lifecycle_state == "ACTIVE":
                            all_subscriptions[topic_id].append(sub.id)
                except Exception as error:
                    logger.error(
                        f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                    )
                    continue

            # Now list all topics and attach their subscriptions
            for compartment in self.audited_compartments:
                try:
                    topics = oci.pagination.list_call_get_all_results(
                        ons_control_client.list_topics, compartment_id=compartment.id
                    ).data

                    for topic in topics:
                        if topic.lifecycle_state != "DELETED":
                            # Get subscriptions for this topic from our pre-fetched map
                            subscriptions = all_subscriptions.get(topic.topic_id, [])

                            self.topics.append(
                                Topic(
                                    id=topic.topic_id,
                                    name=topic.name,
                                    compartment_id=compartment.id,
                                    region=regional_client.region,
                                    lifecycle_state=topic.lifecycle_state,
                                    subscriptions=subscriptions,
                                )
                            )
                except Exception as error:
                    logger.error(
                        f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                    )
                    continue
        except Exception as error:
            logger.error(
                f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )


# Service Models
class Rule(BaseModel):
    """OCI Events Rule model."""

    id: str
    name: str
    compartment_id: str
    region: str
    lifecycle_state: str
    condition: str
    is_enabled: bool
    actions: List = []


class Topic(BaseModel):
    """OCI Notification Topic model."""

    id: str
    name: str
    compartment_id: str
    region: str
    lifecycle_state: str
    subscriptions: List[str] = []



from prowler.lib.check.models import Check, Check_Report_OCI
from prowler.providers.oraclecloud.services.logging.logging_client import logging_client
from prowler.providers.oraclecloud.services.objectstorage.objectstorage_client import (
    objectstorage_client,
)


class objectstorage_bucket_logging_enabled(Check):
    """Ensure write level Object Storage logging is enabled for all buckets"""

    def execute(self):
        """Execute check to verify write-level logging is enabled for Object Storage buckets."""
        findings = []

        for bucket in objectstorage_client.buckets:
            report = Check_Report_OCI(
                metadata=self.metadata(),
                resource=bucket,
                region=bucket.region,
                resource_id=bucket.id,
                resource_name=bucket.name,
                compartment_id=bucket.compartment_id,
            )

            # Check if there is a write-level log configured for this bucket
            # A write log should have:
            # - source.service == 'objectstorage'
            # - source.category == 'write'
            # - source.resource == bucket.name
            has_write_logging = False
            has_read_logging = False
            for log in logging_client.logs:
                if (
                    log.source_service == "objectstorage"
                    and log.source_category == "write"
                    and log.source_resource == bucket.name
                    and log.region == bucket.region
                    and log.is_enabled
                ):
                    has_write_logging = True
                elif (
                    log.source_service == "objectstorage"
                    and log.source_category == "read"
                    and log.source_resource == bucket.name
                    and log.region == bucket.region
                    and log.is_enabled
                ):
                    has_read_logging = True

            if has_write_logging:
                report.status = "PASS"
                report.status_extended = (
                    f"Bucket {bucket.name} has write-level logging enabled."
                )
            elif has_read_logging:
                report.status = "FAIL"
                report.status_extended = (
                    f"Bucket {bucket.name} has read-level logging enabled."
                )
            else:
                report.status = "FAIL"
                report.status_extended = (
                    f"Bucket {bucket.name} does not have write-level logging enabled."
                )

            findings.append(report)

        return findings



"""Check Ensure API keys are not created for tenancy administrator users."""

from prowler.lib.check.models import Check, Check_Report_OCI
from prowler.providers.oraclecloud.services.identity.identity_client import (
    identity_client,
)


class identity_tenancy_admin_users_no_api_keys(Check):
    """Check Ensure API keys are not created for tenancy administrator users."""

    def execute(self) -> Check_Report_OCI:
        """Execute the identity_tenancy_admin_users_no_api_keys check."""
        findings = []

        # Check tenancy admin users for API keys
        for user in identity_client.users:
            # Check if user is in Administrators group
            is_admin = False
            for group_id in user.groups:
                for group in identity_client.groups:
                    if group.id == group_id and "Administrators" in group.name:
                        is_admin = True
                        break

            if is_admin:
                report = Check_Report_OCI(
                    metadata=self.metadata(),
                    resource=user,
                    region=user.region,
                    resource_name=user.name,
                    resource_id=user.id,
                    compartment_id=user.compartment_id,
                )

                if user.api_keys:
                    report.status = "FAIL"
                    report.status_extended = (
                        f"Tenancy administrator user {user.name} has API keys."
                    )
                else:
                    report.status = "PASS"
                    report.status_extended = (
                        f"Tenancy administrator user {user.name} has no API keys."
                    )

                findings.append(report)

        return findings



import re
from abc import ABC, abstractmethod

import yaml
from jsonschema import validate

from prowler.lib.logger import logger
from prowler.lib.outputs.common import Status
from prowler.lib.outputs.utils import unroll_dict, unroll_tags

mutelist_schema = {
    "type": "object",
    "properties": {
        "Accounts": {
            "type": "object",
            "patternProperties": {
                ".*": {  # Match any account
                    "type": "object",
                    "properties": {
                        "Checks": {
                            "type": "object",
                            "patternProperties": {
                                ".*": {  # Match any check
                                    "type": "object",
                                    "properties": {
                                        "Regions": {
                                            "type": "array",
                                            "items": {"type": "string"},
                                        },
                                        "Resources": {
                                            "type": "array",
                                            "items": {"type": "string"},
                                        },
                                        "Tags": {  # Optional field
                                            "type": "array",
                                            "items": {"type": "string"},
                                        },
                                        "Exceptions": {  # Optional field
                                            "type": "object",
                                            "properties": {
                                                "Accounts": {  # Optional field
                                                    "type": "array",
                                                    "items": {"type": "string"},
                                                },
                                                "Regions": {  # Optional field
                                                    "type": "array",
                                                    "items": {"type": "string"},
                                                },
                                                "Resources": {  # Optional field
                                                    "type": "array",
                                                    "items": {"type": "string"},
                                                },
                                                "Tags": {  # Optional field
                                                    "type": "array",
                                                    "items": {"type": "string"},
                                                },
                                            },
                                            "additionalProperties": False,
                                        },
                                        "Description": {  # Optional field
                                            "type": "string",
                                        },
                                    },
                                    "required": [
                                        "Regions",
                                        "Resources",
                                    ],  # Mandatory within a check
                                    "additionalProperties": False,
                                }
                            },
                            "additionalProperties": False,
                        },
                    },
                    "required": ["Checks"],  # Mandatory within an account
                    "additionalProperties": False,
                }
            },
            "additionalProperties": False,
        }
    },
    "required": ["Accounts"],  # Accounts is mandatory at the root level
    "additionalProperties": False,
}


class Mutelist(ABC):
    """
    Abstract base class for managing a mutelist.

    Attributes:
        _mutelist (dict): Dictionary containing information about muted checks for different accounts.
        _mutelist_file_path (str): Path to the mutelist file.
        MUTELIST_KEY (str): Key used to access the mutelist in the mutelist file.

    Methods:
        __init__: Initializes a Mutelist object.
        mutelist: Property that returns the mutelist dictionary.
        mutelist_file_path: Property that returns the mutelist file path.
        is_finding_muted: Abstract method to check if a finding is muted.
        get_mutelist_file_from_local_file: Retrieves the mutelist file from a local file.
        is_muted: Checks if a finding is muted for the audited account, check, region, resource, and tags.
        is_muted_in_check: Checks if a check is muted.
        is_excepted: Checks if the account, region, resource, and tags are excepted based on the exceptions.
    """

    _mutelist: dict = {}
    _mutelist_file_path: str = None

    MUTELIST_KEY = "Mutelist"

    def __init__(
        self, mutelist_path: str = "", mutelist_content: dict = {}
    ) -> "Mutelist":
        if mutelist_path:
            self._mutelist_file_path = mutelist_path
            self.get_mutelist_file_from_local_file(mutelist_path)
        else:
            self._mutelist = mutelist_content

        if self._mutelist:
            self._mutelist = Mutelist.validate_mutelist(self._mutelist)

    @property
    def mutelist(self) -> dict:
        return self._mutelist

    @property
    def mutelist_file_path(self) -> dict:
        return self._mutelist_file_path

    @abstractmethod
    def is_finding_muted(self) -> bool:
        raise NotImplementedError

    def get_mutelist_file_from_local_file(self, mutelist_path: str):
        try:
            with open(mutelist_path) as f:
                self._mutelist = yaml.safe_load(f)[self.MUTELIST_KEY]
        except Exception as error:
            logger.error(
                f"{error.__class__.__name__} -- {error}[{error.__traceback__.tb_lineno}]"
            )

    def is_muted(
        self,
        audited_account: str,
        check: str,
        finding_region: str,
        finding_resource: str,
        finding_tags,
    ) -> bool:
        """
        Check if the provided finding is muted for the audited account, check, region, resource and tags.

        The Mutelist works in a way that each field is ANDed, so if a check is muted for an account, region, resource and tags, it will be muted.

        Exceptions use AND logic across specified fields, with unspecified fields treated as wildcards (matching all values).

        Tag matching uses AND logic when multiple tags are listed (all must match). OR logic is achieved using regex alternation (|) within a single tag pattern.

        So, for the following Mutelist:
        ```
        Mutelist:
            Accounts:
                '*':
                Checks:
                    ec2_instance_detailed_monitoring_enabled:
                        Regions: ['*']
                        Resources:
                            - 'i-123456789'
                        Tags:
                            - 'Name=AdminInstance|Environment=Prod'
                        Description: 'Field to describe why the findings associated with these values are muted'
        ```
        The check `ec2_instance_detailed_monitoring_enabled` will be muted for all accounts and regions and for the resource_id 'i-123456789' with at least one of the tags 'Name=AdminInstance' or 'Environment=Prod'.

        Note: The pipe (|) in the tag pattern provides OR logic via regex alternation. To require BOTH tags, use two separate tag entries:
        Tags:
            - 'Name=AdminInstance'
            - 'Environment=Prod'

        Args:
            mutelist (dict): Dictionary containing information about muted checks for different accounts.
            audited_account (str): The account being audited.
            check (str): The check to be evaluated for muting.
            finding_region (str): The region where the finding occurred.
            finding_resource (str): The resource related to the finding.
            finding_tags: The tags associated with the finding.

        Returns:
            bool: True if the finding is muted for the audited account, check, region, resource and tags., otherwise False.
        """
        try:
            # By default is not muted
            is_finding_muted = False

            # We always check all the accounts present in the mutelist
            # if one mutes the finding we set the finding as muted
            for account in self._mutelist.get("Accounts", []):
                if account == audited_account or account == "*":
                    if self.is_muted_in_check(
                        self._mutelist["Accounts"][account]["Checks"],
                        audited_account,
                        check,
                        finding_region,
                        finding_resource,
                        finding_tags,
                    ):
                        is_finding_muted = True
                        break

            return is_finding_muted
        except Exception as error:
            logger.error(
                f"{error.__class__.__name__} -- {error}[{error.__traceback__.tb_lineno}]"
            )
            return False

    def is_muted_in_check(
        self,
        muted_checks,
        audited_account,
        check,
        finding_region,
        finding_resource,
        finding_tags,
    ) -> bool:
        """
        Check if the provided check is muted.

        Args:
            muted_checks (dict): Dictionary containing information about muted checks.
            audited_account (str): The account to be audited.
            check (str): The check to be evaluated for muting.
            finding_region (str): The region where the finding occurred.
            finding_resource (str): The resource related to the finding.
            finding_tags (str): The tags associated with the finding.

        Returns:
            bool: True if the check is muted, otherwise False.
        """
        try:
            # Default value is not muted
            is_check_muted = False

            for muted_check, muted_check_info in muted_checks.items():
                # map lambda to awslambda
                muted_check = re.sub("^lambda", "awslambda", muted_check)

                check_match = (
                    "*" == muted_check
                    or check == muted_check
                    or self.is_item_matched([muted_check], check)
                )

                # Check if the finding is excepted
                exceptions = muted_check_info.get("Exceptions")
                if (
                    self.is_excepted(
                        exceptions,
                        audited_account,
                        finding_region,
                        finding_resource,
                        finding_tags,
                    )
                    and check_match
                ):
                    # Break loop and return default value since is excepted
                    break

                muted_regions = muted_check_info.get("Regions")
                muted_resources = muted_check_info.get("Resources")
                muted_tags = muted_check_info.get("Tags", "*")
                # We need to set the muted_tags if None, "" or [], so the falsy helps
                if not muted_tags:
                    muted_tags = "*"
                # If there is a *, it affects to all checks
                if check_match:
                    muted_in_check = True
                    muted_in_region = self.is_item_matched(
                        muted_regions, finding_region
                    )
                    muted_in_resource = self.is_item_matched(
                        muted_resources, finding_resource
                    )
                    muted_in_tags = self.is_item_matched(
                        muted_tags, finding_tags, tag=True
                    )

                    # For a finding to be muted requires the following set to True:
                    # - muted_in_check -> True
                    # - muted_in_region -> True
                    # - muted_in_tags -> True
                    # - muted_in_resource -> True
                    # - excepted -> False

                    if (
                        muted_in_check
                        and muted_in_region
                        and muted_in_tags
                        and muted_in_resource
                    ):
                        is_check_muted = True

            return is_check_muted
        except Exception as error:
            logger.error(
                f"{error.__class__.__name__} -- {error}[{error.__traceback__.tb_lineno}]"
            )
            return False

    def mute_finding(self, finding):
        """
        Check if the provided finding is muted

        Args:
            finding (Finding): The finding to be evaluated for muting.

        Returns:
            Finding: The finding with the status updated if it is muted, otherwise the finding is returned

        """
        try:
            if self.is_muted(
                finding.account_uid,
                finding.metadata.CheckID,
                finding.region,
                finding.resource_uid,
                unroll_dict(unroll_tags(finding.resource_tags)),
            ):
                finding.raw["status"] = finding.status
                finding.status = Status.MUTED
                finding.muted = True
            return finding
        except Exception as error:
            logger.error(
                f"{error.__class__.__name__} -- {error}[{error.__traceback__.tb_lineno}]"
            )
            return finding

    def is_excepted(
        self,
        exceptions,
        audited_account,
        finding_region,
        finding_resource,
        finding_tags,
    ) -> bool:
        """
        Check if the provided account, region, resource, and tags are excepted based on the exceptions dictionary.

        Args:
            exceptions (dict): Dictionary containing exceptions for different attributes like Accounts, Regions, Resources, and Tags.
            audited_account (str): The account to be audited.
            finding_region (str): The region where the finding occurred.
            finding_resource (str): The resource related to the finding.
            finding_tags (str): The tags associated with the finding.

        Returns:
            bool: True if the account, region, resource, and tags are excepted based on the exceptions, otherwise False.
        """
        try:
            excepted = False
            is_account_excepted = False
            is_region_excepted = False
            is_resource_excepted = False
            is_tag_excepted = False
            if exceptions:
                excepted_accounts = exceptions.get("Accounts", [])
                is_account_excepted = self.is_item_matched(
                    excepted_accounts, audited_account
                )

                excepted_regions = exceptions.get("Regions", [])
                is_region_excepted = self.is_item_matched(
                    excepted_regions, finding_region
                )

                excepted_resources = exceptions.get("Resources", [])
                is_resource_excepted = self.is_item_matched(
                    excepted_resources, finding_resource
                )

                excepted_tags = exceptions.get("Tags", [])
                is_tag_excepted = self.is_item_matched(
                    excepted_tags, finding_tags, tag=True
                )

                if (
                    not is_account_excepted
                    and not is_region_excepted
                    and not is_resource_excepted
                    and not is_tag_excepted
                ):
                    excepted = False
                elif (
                    (is_account_excepted or not excepted_accounts)
                    and (is_region_excepted or not excepted_regions)
                    and (is_resource_excepted or not excepted_resources)
                    and (is_tag_excepted or not excepted_tags)
                ):
                    excepted = True
            return excepted
        except Exception as error:
            logger.error(
                f"{error.__class__.__name__} -- {error}[{error.__traceback__.tb_lineno}]"
            )
            return False

    @staticmethod
    def is_item_matched(matched_items, finding_items, tag=False) -> bool:
        """
        Check if any of the items in matched_items are present in finding_items.

        Args:
            matched_items (list): List of items to be matched.
            finding_items (str): String to search for matched items.
            tag (bool): If True, uses AND logic across multiple tags in the list.
                - Multiple tags: ALL tags in matched_items must be present in finding_items (AND logic).
                - Single tag with regex alternation (|): Matches if pattern is found (enables OR within pattern).
                - For non-tags: Uses OR logic - returns True if ANY item matches.

        Returns:
            bool: For tags - True if ALL patterns match. For non-tags - True if ANY pattern matches.
        """
        try:
            is_item_matched = False
            if matched_items and (finding_items or finding_items == ""):
                if tag:
                    is_item_matched = True
                for item in matched_items:
                    if "*" in item:
                        item = item.replace("*", ".*")
                    if tag:
                        if not re.search(item, finding_items):
                            is_item_matched = False
                            break
                    else:
                        if re.search(item, finding_items):
                            is_item_matched = True
                            break
            return is_item_matched
        except Exception as error:
            logger.error(
                f"{error.__class__.__name__} -- {error}[{error.__traceback__.tb_lineno}]"
            )
            return False

    @staticmethod
    def validate_mutelist(mutelist: dict, raise_on_exception: bool = False) -> dict:
        """
        Validate the mutelist against the schema.

        Args:
            mutelist (dict): The mutelist to be validated.
            raise_on_exception (bool): Whether to raise an exception if the mutelist is invalid.

        Returns:
            dict: The mutelist itself.
        """
        try:
            validate(mutelist, schema=mutelist_schema)
            return mutelist
        except Exception as error:
            if raise_on_exception:
                raise error
            else:
                logger.error(
                    f"{error.__class__.__name__} -- Mutelist YAML is malformed - {error}[{error.__traceback__.tb_lineno}]"
                )
            return {}



from typing import Optional

from googleapiclient.errors import HttpError
from pydantic.v1 import BaseModel

from prowler.lib.logger import logger
from prowler.providers.gcp.config import DEFAULT_RETRY_ATTEMPTS
from prowler.providers.gcp.gcp_provider import GcpProvider
from prowler.providers.gcp.lib.service.service import GCPService


class CloudStorage(GCPService):
    def __init__(self, provider: GcpProvider):
        super().__init__("storage", provider)
        self.buckets = []
        self.vpc_service_controls_protected_projects = set()
        self._get_buckets()

    def _get_buckets(self):
        for project_id in self.project_ids:
            try:
                request = self.client.buckets().list(project=project_id)
                while request is not None:
                    response = request.execute(num_retries=DEFAULT_RETRY_ATTEMPTS)
                    for bucket in response.get("items", []):
                        bucket_iam = (
                            self.client.buckets()
                            .getIamPolicy(bucket=bucket["id"])
                            .execute(num_retries=DEFAULT_RETRY_ATTEMPTS)["bindings"]
                        )
                        public = False
                        if "allAuthenticatedUsers" in str(
                            bucket_iam
                        ) or "allUsers" in str(bucket_iam):
                            public = True

                        lifecycle_rules = None
                        lifecycle = bucket.get("lifecycle")
                        if isinstance(lifecycle, dict):
                            rules = lifecycle.get("rule")
                            if isinstance(rules, list):
                                lifecycle_rules = rules

                        versioning_enabled = bucket.get("versioning", {}).get(
                            "enabled", False
                        )

                        soft_delete_enabled = False
                        soft_delete_policy = bucket.get("softDeletePolicy")
                        if isinstance(soft_delete_policy, dict):
                            retention = soft_delete_policy.get(
                                "retentionDurationSeconds"
                            )
                            if retention and int(retention) > 0:
                                soft_delete_enabled = True

                        logging_info = bucket.get("logging", {})
                        logging_bucket = logging_info.get("logBucket")
                        logging_prefix = logging_info.get("logObjectPrefix")

                        retention_policy_raw = bucket.get("retentionPolicy")
                        retention_policy = None
                        if isinstance(retention_policy_raw, dict):
                            rp_seconds = retention_policy_raw.get("retentionPeriod")
                            if rp_seconds:
                                retention_policy = RetentionPolicy(
                                    retention_period=int(rp_seconds),
                                    is_locked=bool(
                                        retention_policy_raw.get("isLocked", False)
                                    ),
                                    effective_time=retention_policy_raw.get(
                                        "effectiveTime"
                                    ),
                                )

                        self.buckets.append(
                            Bucket(
                                name=bucket["name"],
                                id=bucket["id"],
                                region=bucket["location"],
                                uniform_bucket_level_access=bucket["iamConfiguration"][
                                    "uniformBucketLevelAccess"
                                ]["enabled"],
                                public=public,
                                retention_policy=retention_policy,
                                project_id=project_id,
                                lifecycle_rules=lifecycle_rules,
                                versioning_enabled=versioning_enabled,
                                soft_delete_enabled=soft_delete_enabled,
                                logging_bucket=logging_bucket,
                                logging_prefix=logging_prefix,
                            )
                        )

                    request = self.client.buckets().list_next(
                        previous_request=request, previous_response=response
                    )
            except HttpError as http_error:
                # Check if the error is due to VPC Service Controls blocking the API
                if "vpcServiceControlsUniqueIdentifier" in str(http_error):
                    self.vpc_service_controls_protected_projects.add(project_id)
                    logger.warning(
                        f"Project {project_id} is protected by VPC Service Controls for Cloud Storage API."
                    )
                else:
                    logger.error(
                        f"{http_error.__class__.__name__}[{http_error.__traceback__.tb_lineno}]: {http_error}"
                    )
            except Exception as error:
                logger.error(
                    f"{error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                )


class RetentionPolicy(BaseModel):
    retention_period: int
    is_locked: bool
    effective_time: Optional[str] = None


class Bucket(BaseModel):
    name: str
    id: str
    region: str
    uniform_bucket_level_access: bool
    public: bool
    project_id: str
    retention_policy: Optional[RetentionPolicy] = None
    lifecycle_rules: Optional[list[dict]] = None
    versioning_enabled: Optional[bool] = False
    soft_delete_enabled: Optional[bool] = False
    logging_bucket: Optional[str] = None
    logging_prefix: Optional[str] = None



from typing import List

from prowler.lib.check.models import Check, CheckReportMongoDBAtlas
from prowler.providers.mongodbatlas.services.clusters.clusters_client import (
    clusters_client,
)


class clusters_backup_enabled(Check):
    """Check if MongoDB Atlas clusters have backup enabled

    This class verifies that MongoDB Atlas clusters have backup enabled
    to protect against data loss.
    """

    def execute(self) -> List[CheckReportMongoDBAtlas]:
        """Execute the MongoDB Atlas cluster backup enabled check

        Iterates over all clusters and checks if they have backup
        enabled (backupEnabled=true).

        Returns:
            List[CheckReportMongoDBAtlas]: A list of reports for each cluster
        """
        findings = []

        for cluster in clusters_client.clusters.values():
            report = CheckReportMongoDBAtlas(metadata=self.metadata(), resource=cluster)

            if cluster.backup_enabled:
                report.status = "PASS"
                report.status_extended = (
                    f"Cluster {cluster.name} in project {cluster.project_name} "
                    f"has backup enabled."
                )
            else:
                report.status = "FAIL"
                report.status_extended = (
                    f"Cluster {cluster.name} in project {cluster.project_name} "
                    f"does not have backup enabled."
                )

            findings.append(report)

        return findings



def init_parser(self):
    """Initialize the MongoDB Atlas Provider CLI parser"""
    mongodbatlas_parser = self.subparsers.add_parser(
        "mongodbatlas",
        parents=[self.common_providers_parser],
        help="MongoDB Atlas Provider",
    )

    mongodbatlas_auth_subparser = mongodbatlas_parser.add_argument_group(
        "Authentication Modes"
    )

    mongodbatlas_auth_subparser.add_argument(
        "--atlas-public-key",
        nargs="?",
        help="MongoDB Atlas API public key",
        default=None,
        metavar="ATLAS_PUBLIC_KEY",
    )

    mongodbatlas_auth_subparser.add_argument(
        "--atlas-private-key",
        nargs="?",
        help="MongoDB Atlas API private key",
        default=None,
        metavar="ATLAS_PRIVATE_KEY",
    )

    mongodbatlas_filters_subparser = mongodbatlas_parser.add_argument_group(
        "Optional Filters"
    )

    mongodbatlas_filters_subparser.add_argument(
        "--atlas-project-id",
        nargs="?",
        help="MongoDB Atlas Project ID to filter scans to a specific project",
        default=None,
        metavar="ATLAS_PROJECT_ID",
    )


def validate_arguments(arguments):
    """Validate MongoDB Atlas provider arguments"""
    # No specific validation needed for MongoDB Atlas arguments currently
    return (True, "")



from datetime import datetime
from typing import Optional

from alibabacloud_actiontrail20200706.client import Client as ActionTrailClient
from alibabacloud_cs20151215.client import Client as CSClient
from alibabacloud_ecs20140526.client import Client as EcsClient
from alibabacloud_oss20190517.client import Client as OssClient
from alibabacloud_ram20150501.client import Client as RamClient
from alibabacloud_rds20140815.client import Client as RdsClient
from alibabacloud_sas20181203.client import Client as SasClient
from alibabacloud_sls20201230.client import Client as SlsClient
from alibabacloud_tea_openapi import models as open_api_models
from alibabacloud_vpc20160428.client import Client as VpcClient
from pydantic.v1 import BaseModel

from prowler.lib.logger import logger
from prowler.providers.alibabacloud.config import (
    ALIBABACLOUD_DEFAULT_REGION,
    ALIBABACLOUD_SDK_CONNECT_TIMEOUT,
    ALIBABACLOUD_SDK_READ_TIMEOUT,
)
from prowler.providers.common.models import ProviderOutputOptions


class AlibabaCloudCallerIdentity(BaseModel):
    """
    AlibabaCloudCallerIdentity stores the caller identity information from STS GetCallerIdentity.

    Attributes:
        account_id: The Alibaba Cloud account ID
        principal_id: The principal ID (user ID or root account ID)
        arn: The ARN-like identifier for the identity
        identity_type: The type of identity (e.g., "RamUser", "Root")
    """

    account_id: str
    principal_id: str
    arn: str
    identity_type: str = ""


class AlibabaCloudIdentityInfo(BaseModel):
    """
    AlibabaCloudIdentityInfo stores the Alibaba Cloud account identity information.

    Attributes:
        account_id: The Alibaba Cloud account ID
        account_name: The Alibaba Cloud account name (if available)
        user_id: The RAM user ID or root account ID
        user_name: The RAM user name or "root" for root account
        identity_arn: The ARN-like identifier for the identity
        profile: The profile name used for authentication
        profile_region: The default region from the profile
        audited_regions: Set of regions to be audited
        is_root: Whether this is the root account (True) or a RAM user (False)
    """

    account_id: str
    account_name: str
    user_id: str
    user_name: str
    identity_arn: str
    profile: str
    profile_region: str
    audited_regions: set[str]
    is_root: bool = False


class AlibabaCloudCredentials(BaseModel):
    """
    AlibabaCloudCredentials stores the Alibaba Cloud credentials.

    Attributes:
        access_key_id: The Access Key ID
        access_key_secret: The Access Key Secret
        security_token: The Security Token (for STS temporary credentials)
        expiration: The expiration time for temporary credentials
    """

    access_key_id: str
    access_key_secret: str
    security_token: Optional[str] = None
    expiration: Optional[datetime] = None


class AlibabaCloudAssumeRoleInfo(BaseModel):
    """
    AlibabaCloudAssumeRoleInfo stores the information for assuming a RAM role.

    Attributes:
        role_arn: The ARN of the role to assume
        role_session_name: The session name for the assumed role
        session_duration: The duration of the assumed role session (in seconds)
        external_id: The external ID for role assumption
        region: The region for STS endpoint
    """

    role_arn: str
    role_session_name: str
    session_duration: int
    external_id: Optional[str] = None
    region: str = "cn-hangzhou"


class AlibabaCloudRegion(BaseModel):
    """
    AlibabaCloudRegion stores information about an Alibaba Cloud region.

    Attributes:
        region_id: The region identifier (e.g., cn-hangzhou, cn-shanghai)
        region_name: The human-readable region name
        region_endpoint: The API endpoint for the region
    """

    region_id: str
    region_name: str
    region_endpoint: Optional[str] = None


class AlibabaCloudSession:
    """
    AlibabaCloudSession stores the Alibaba Cloud session and credentials.

    This class provides methods to get credentials and create service clients.
    """

    def __init__(self, cred_client):
        """
        Initialize the Alibaba Cloud session.

        Args:
            cred_client: The Alibaba Cloud credentials client
        """
        self.cred_client = cred_client
        self._credentials = None

    def get_credentials(self):
        """
        Get the Alibaba Cloud credentials.

        Returns:
            AlibabaCloudCredentials object
        """
        if self._credentials is None:
            cred = self.cred_client.get_credential()
            self._credentials = AlibabaCloudCredentials(
                access_key_id=cred.get_access_key_id(),
                access_key_secret=cred.get_access_key_secret(),
                security_token=cred.get_security_token(),
            )
        return self._credentials

    def client(self, service: str, region: str = None):
        """
        Create a service client for the given service and region.

        Args:
            service: The service name (e.g., 'ram')
            region: The region (optional, some services are global)

        Returns:
            A client instance for the specified service
        """

        # Get credentials
        cred = self.get_credentials()

        # Create client configuration with timeout settings
        config = open_api_models.Config(
            access_key_id=cred.access_key_id,
            access_key_secret=cred.access_key_secret,
            read_timeout=ALIBABACLOUD_SDK_READ_TIMEOUT
            * 1000,  # Convert to milliseconds
            connect_timeout=ALIBABACLOUD_SDK_CONNECT_TIMEOUT
            * 1000,  # Convert to milliseconds
        )
        if cred.security_token:
            config.security_token = cred.security_token

        # Set endpoint based on service
        if service == "ram":
            config.endpoint = "ram.aliyuncs.com"
            return RamClient(config)
        elif service == "vpc":
            # VPC endpoint is regional: vpc.{region}.aliyuncs.com
            if region:
                config.endpoint = f"vpc.{region}.aliyuncs.com"
            else:
                config.endpoint = f"vpc.{ALIBABACLOUD_DEFAULT_REGION}.aliyuncs.com"
            return VpcClient(config)
        elif service == "ecs":
            # ECS endpoint is regional: ecs.{region}.aliyuncs.com
            if region:
                config.endpoint = f"ecs.{region}.aliyuncs.com"
            else:
                config.endpoint = f"ecs.{ALIBABACLOUD_DEFAULT_REGION}.aliyuncs.com"
            return EcsClient(config)
        elif service == "sas" or service == "securitycenter":
            # SAS (Security Center) endpoint is regional: sas.{region}.aliyuncs.com
            if region:
                config.endpoint = f"sas.{region}.aliyuncs.com"
            else:
                config.endpoint = f"sas.{ALIBABACLOUD_DEFAULT_REGION}.aliyuncs.com"
            return SasClient(config)
        elif service == "oss":
            if region:
                config.endpoint = f"oss-{region}.aliyuncs.com"
                config.region_id = region
            else:
                config.endpoint = f"oss-{ALIBABACLOUD_DEFAULT_REGION}.aliyuncs.com"
                config.region_id = ALIBABACLOUD_DEFAULT_REGION
            return OssClient(config)
        elif service == "actiontrail":
            # ActionTrail endpoint is regional: actiontrail.{region}.aliyuncs.com
            if region:
                config.endpoint = f"actiontrail.{region}.aliyuncs.com"
            else:
                config.endpoint = (
                    f"actiontrail.{ALIBABACLOUD_DEFAULT_REGION}.aliyuncs.com"
                )
            return ActionTrailClient(config)
        elif service == "cs":
            if region:
                config.endpoint = f"cs.{region}.aliyuncs.com"
            else:
                config.endpoint = f"cs.{ALIBABACLOUD_DEFAULT_REGION}.aliyuncs.com"
            return CSClient(config)
        elif service == "rds":
            if region:
                config.endpoint = f"rds.{region}.aliyuncs.com"
            else:
                config.endpoint = f"rds.{ALIBABACLOUD_DEFAULT_REGION}.aliyuncs.com"
            return RdsClient(config)
        elif service == "sls":
            if region:
                config.endpoint = f"{region}.log.aliyuncs.com"
            else:
                config.endpoint = f"{ALIBABACLOUD_DEFAULT_REGION}.log.aliyuncs.com"
            return SlsClient(config)
        else:
            # For other services, implement as needed
            logger.warning(f"Service {service} not yet implemented")
            return None


class AlibabaCloudOutputOptions(ProviderOutputOptions):
    """
    AlibabaCloudOutputOptions extends ProviderOutputOptions for Alibaba Cloud specific output options.
    """

    def __init__(self, arguments, bulk_checks_metadata, identity):
        # Call parent class init
        super().__init__(arguments, bulk_checks_metadata)

        # Set default output filename if not provided
        if (
            not hasattr(arguments, "output_filename")
            or arguments.output_filename is None
        ):
            from prowler.config.config import output_file_timestamp

            self.output_filename = (
                f"prowler-output-{identity.account_id}-{output_file_timestamp}"
            )
        else:
            self.output_filename = arguments.output_filename



from prowler.providers.common.provider import Provider
from prowler.providers.oraclecloud.services.identity.identity_service import Identity

identity_client = Identity(Provider.get_global_provider())



from prowler.lib.check.models import Check, CheckReportAlibabaCloud
from prowler.providers.alibabacloud.services.cs.cs_client import cs_client


class cs_kubernetes_eni_multiple_ip_enabled(Check):
    """Check if ENI multiple IP mode is supported on Kubernetes Engine Clusters."""

    def execute(self) -> list[CheckReportAlibabaCloud]:
        findings = []

        for cluster in cs_client.clusters:
            report = CheckReportAlibabaCloud(metadata=self.metadata(), resource=cluster)
            report.region = cluster.region
            report.resource_id = cluster.id
            report.resource_arn = f"acs:cs:{cluster.region}:{cs_client.audited_account}:cluster/{cluster.id}"

            if cluster.eni_multiple_ip_enabled:
                report.status = "PASS"
                report.status_extended = f"Kubernetes cluster {cluster.name} supports ENI multiple IP mode via Terway plugin."
            else:
                report.status = "FAIL"
                report.status_extended = f"Kubernetes cluster {cluster.name} does not support ENI multiple IP mode (requires Terway network plugin)."

            findings.append(report)

        return findings



"""OCI Filestorage Service Module."""

from typing import Optional

import oci
from pydantic import BaseModel

from prowler.lib.logger import logger
from prowler.providers.oraclecloud.lib.service.service import OCIService


class Filestorage(OCIService):
    """OCI Filestorage Service class."""

    def __init__(self, provider):
        """Initialize the Filestorage service."""
        super().__init__("filestorage", provider)
        self.file_systems = []
        self.__threading_call__(self.__list_file_systems__)

    def __get_client__(self, region):
        """Get the Filestorage client for a region."""
        client_region = self.regional_clients.get(region)
        if client_region:
            return self._create_oci_client(oci.file_storage.FileStorageClient)
        return None

    def __list_file_systems__(self, regional_client):
        """List all file_systems."""
        try:
            client = self.__get_client__(regional_client.region)
            if not client:
                return

            logger.info(
                f"Filestorage - Listing file_systems in {regional_client.region}..."
            )

            for compartment in self.audited_compartments:
                try:
                    # Get availability domains for this compartment
                    identity_client = self._create_oci_client(
                        oci.identity.IdentityClient
                    )
                    availability_domains = identity_client.list_availability_domains(
                        compartment_id=compartment.id
                    ).data

                    # List file systems in each availability domain
                    for ad in availability_domains:
                        items = oci.pagination.list_call_get_all_results(
                            client.list_file_systems,
                            compartment_id=compartment.id,
                            availability_domain=ad.name,
                        ).data

                        for item in items:
                            if item.lifecycle_state not in ["DELETED", "DELETING"]:
                                self.file_systems.append(
                                    FileSystem(
                                        id=item.id,
                                        name=(
                                            item.display_name
                                            if hasattr(item, "display_name")
                                            else item.id
                                        ),
                                        compartment_id=compartment.id,
                                        region=regional_client.region,
                                        lifecycle_state=item.lifecycle_state,
                                        kms_key_id=(
                                            item.kms_key_id
                                            if hasattr(item, "kms_key_id")
                                            else None
                                        ),
                                    )
                                )
                except Exception as error:
                    logger.error(
                        f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                    )
                    continue
        except Exception as error:
            logger.error(
                f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )


class FileSystem(BaseModel):
    """FileSystem model."""

    id: str
    name: str
    compartment_id: str
    region: str
    lifecycle_state: str
    kms_key_id: Optional[str] = None



import os
from concurrent.futures import ThreadPoolExecutor, as_completed
from dataclasses import dataclass
from typing import Optional, Union

from boto3 import Session
from botocore.client import ClientError
from botocore.exceptions import NoCredentialsError, ProfileNotFound

from prowler.config.config import timestamp_utc
from prowler.lib.logger import logger
from prowler.lib.outputs.asff.asff import AWSSecurityFindingFormat
from prowler.providers.aws.aws_provider import AwsProvider
from prowler.providers.aws.config import (
    AWS_STS_GLOBAL_ENDPOINT_REGION,
    ROLE_SESSION_NAME,
)
from prowler.providers.aws.exceptions.exceptions import (
    AWSAccessKeyIDInvalidError,
    AWSArgumentTypeValidationError,
    AWSAssumeRoleError,
    AWSIAMRoleARNEmptyResourceError,
    AWSIAMRoleARNInvalidAccountIDError,
    AWSIAMRoleARNInvalidResourceTypeError,
    AWSIAMRoleARNPartitionEmptyError,
    AWSIAMRoleARNRegionNotEmtpyError,
    AWSIAMRoleARNServiceNotIAMnorSTSError,
    AWSNoCredentialsError,
    AWSProfileNotFoundError,
    AWSSecretAccessKeyInvalidError,
    AWSSessionTokenExpiredError,
    AWSSetUpSessionError,
)
from prowler.providers.aws.lib.arguments.arguments import (
    validate_role_session_name,
    validate_session_duration,
)
from prowler.providers.aws.lib.arn.arn import parse_iam_credentials_arn
from prowler.providers.aws.lib.security_hub.exceptions.exceptions import (
    SecurityHubInvalidRegionError,
    SecurityHubNoEnabledRegionsError,
)
from prowler.providers.aws.lib.session.aws_set_up_session import AwsSetUpSession
from prowler.providers.aws.models import AWSAssumeRoleInfo
from prowler.providers.common.models import Connection

SECURITY_HUB_INTEGRATION_NAME = "prowler/prowler"
SECURITY_HUB_MAX_BATCH = 100


@dataclass
class SecurityHubConnection(Connection):
    """
    Represents a Security Hub connection object.
    Attributes:
        enabled_regions (set): Set of regions where Security Hub is enabled.
        disabled_regions (set): Set of regions where Security Hub is disabled.
        partition (str): AWS partition (e.g., aws, aws-cn, aws-us-gov) where SecurityHub is deployed.
    """

    enabled_regions: set = None
    disabled_regions: set = None
    partition: str = ""


class SecurityHub:
    """
    Class representing a SecurityHub object for managing findings and interactions with AWS Security Hub.

    Attributes:
        _session (Session): AWS session object for authentication and communication with AWS services.
        _aws_account_id (str): AWS account ID associated with the SecurityHub instance.
        _aws_partition (str): AWS partition (e.g., aws, aws-cn, aws-us-gov) where SecurityHub is deployed.
        _findings_per_region (dict): Dictionary containing findings per region.
        _enabled_regions (dict): Dictionary containing enabled regions with SecurityHub clients.

    Methods:
        __init__: Initializes the SecurityHub object with necessary attributes.
        filter: Filters findings based on region, returning a dictionary with findings per region.
        verify_enabled_per_region: Verifies and stores enabled regions with SecurityHub clients.
        batch_send_to_security_hub: Sends findings to Security Hub and returns the count of successfully sent findings.
        archive_previous_findings: Archives findings that are not present in the current execution.
        _send_findings_to_security_hub: Sends findings to AWS Security Hub in batches and returns the count of successfully sent findings.
    """

    _session: Session
    _aws_account_id: str
    _aws_partition: str
    _findings_per_region: dict[str, list[AWSSecurityFindingFormat]]
    _enabled_regions: dict[str, Session]

    def __init__(
        self,
        aws_account_id: str,
        aws_partition: str = None,
        aws_session: Session = None,
        findings: list[AWSSecurityFindingFormat] = [],
        aws_security_hub_available_regions: list[str] = [],
        send_only_fails: bool = False,
        role_arn: str = None,
        session_duration: int = 3600,
        external_id: str = None,
        role_session_name: str = ROLE_SESSION_NAME,
        mfa: bool = None,
        profile: str = None,
        aws_access_key_id: str = None,
        aws_secret_access_key: str = None,
        aws_session_token: Optional[str] = None,
        retries_max_attempts: int = 3,
        regions: set = set(),
    ) -> "SecurityHub":
        """
        Initializes the SecurityHub object with the necessary attributes.

        Args:
        - aws_session (Session): AWS session object for authentication and communication with AWS services.
        - aws_account_id (str): AWS account ID associated with the SecurityHub instance.
        - aws_partition (str): AWS partition (e.g., aws, aws-cn, aws-us-gov) where SecurityHub is deployed.
        - findings (list[AWSSecurityFindingFormat]): List of findings to filter and send to Security Hub.
        - aws_security_hub_available_regions (list[str]): List of regions where Security Hub is available.
        - send_only_fails (bool): Flag indicating whether to send only findings with status 'FAIL'.
        - role_arn: The ARN of the IAM role to assume.
        - session_duration: The duration of the session in seconds, between 900 and 43200.
        - external_id: The external ID to use when assuming the IAM role.
        - role_session_name: The name of the session when assuming the IAM role.
        - mfa: A boolean indicating whether MFA is enabled.
        - profile: The name of the AWS CLI profile to use.
        - aws_access_key_id: The AWS access key ID.
        - aws_secret_access_key: The AWS secret access key.
        - aws_session_token: The AWS session token, optional.
        - retries_max_attempts: The maximum number of retries for the AWS client.
        - regions: A set of regions to audit.
        """
        if aws_session:
            self._session = aws_session
        else:
            aws_setup_session = AwsSetUpSession(
                role_arn=role_arn,
                session_duration=session_duration,
                external_id=external_id,
                role_session_name=role_session_name,
                mfa=mfa,
                profile=profile,
                aws_access_key_id=aws_access_key_id,
                aws_secret_access_key=aws_secret_access_key,
                aws_session_token=aws_session_token,
                retries_max_attempts=retries_max_attempts,
                regions=regions,
            )
            self._session = aws_setup_session._session.current_session
        self._aws_account_id = aws_account_id
        if not aws_partition:
            aws_partition = AwsProvider.validate_credentials(
                self._session, AWS_STS_GLOBAL_ENDPOINT_REGION
            ).arn.partition
        self._aws_partition = aws_partition

        self._enabled_regions = None
        self._findings_per_region = {}

        if aws_security_hub_available_regions:
            self._enabled_regions = self.verify_enabled_per_region(
                aws_security_hub_available_regions,
                self._session,
                aws_account_id,
                aws_partition,
            )
        if findings:
            if not self._enabled_regions:
                logger.error("No enabled regions found in Security Hub.")
            else:
                self._findings_per_region = self.filter(findings, send_only_fails)

    def filter(
        self,
        findings: list[AWSSecurityFindingFormat],
        send_only_fails: bool,
    ) -> dict:
        """
        Filters the given list of findings based on the provided criteria and returns a dictionary containing findings per region.

        Args:
            findings (list[AWSSecurityFindingFormat]): List of findings to filter.
            send_only_fails (bool): Flag indicating whether to send only findings with status 'FAIL'.

        Returns:
            dict: A dictionary containing findings per region after applying the filtering criteria.
        """

        findings_per_region = {}
        disabled_regions_logged = set()
        try:
            # Create a key per audited region
            for region in self._enabled_regions.keys():
                findings_per_region[region] = []

            for finding in findings:
                # We don't send findings to not enabled regions
                if finding.Resources[0].Region not in findings_per_region:
                    # Only log once per disabled region
                    if finding.Resources[0].Region not in disabled_regions_logged:
                        logger.warning(
                            f"Skipping findings in region {finding.Resources[0].Region} because it is not enabled in Security Hub."
                        )
                        disabled_regions_logged.add(finding.Resources[0].Region)
                    continue

                if (
                    finding.Compliance.Status != "FAILED"
                    or finding.Compliance.Status == "WARNING"
                ) and send_only_fails:
                    continue

                # Get the finding region
                # We can do that since the finding always stores just one finding
                region = finding.Resources[0].Region

                # Include that finding within their region
                findings_per_region[region].append(finding)
        except Exception as error:
            logger.error(
                f"{error.__class__.__name__} -- [{error.__traceback__.tb_lineno}]: {error}"
            )
        return findings_per_region

    @staticmethod
    def _check_region_security_hub(
        region: str,
        session: Session,
        aws_account_id: str,
        aws_partition: str,
    ) -> tuple[str, Union[Session, None]]:
        """
        Check if Security Hub is enabled in a specific region and if Prowler integration is active.

        Args:
            region (str): AWS region to check.
            session (Session): AWS session object.
            aws_account_id (str): AWS account ID.
            aws_partition (str): AWS partition.

        Returns:
            tuple: (region, client or None) - Returns client if enabled, None otherwise.
        """
        try:
            logger.info(
                f"Checking if the {SECURITY_HUB_INTEGRATION_NAME} is enabled in the {region} region."
            )
            # Check if security hub is enabled in current region
            security_hub_client = session.client("securityhub", region_name=region)
            security_hub_client.describe_hub()

            # Check if Prowler integration is enabled in Security Hub
            security_hub_prowler_integration_arn = f"arn:{aws_partition}:securityhub:{region}:{aws_account_id}:product-subscription/{SECURITY_HUB_INTEGRATION_NAME}"
            if security_hub_prowler_integration_arn not in str(
                security_hub_client.list_enabled_products_for_import()
            ):
                logger.warning(
                    f"Security Hub is enabled in {region} but Prowler integration does not accept findings. More info: https://docs.prowler.com/user-guide/providers/aws/securityhub#aws-security-hub-integration-with-prowler"
                )
                return region, None
            else:
                return region, session.client("securityhub", region_name=region)

        # Handle all the permissions / configuration errors
        except ClientError as client_error:
            # Check if Account is subscribed to Security Hub
            error_code = client_error.response["Error"]["Code"]
            error_message = client_error.response["Error"]["Message"]
            if (
                error_code == "InvalidAccessException"
                and f"Account {aws_account_id} is not subscribed to AWS Security Hub"
                in error_message
            ):
                logger.warning(
                    f"{client_error.__class__.__name__} -- [{client_error.__traceback__.tb_lineno}]: {client_error}"
                )
            else:
                logger.error(
                    f"{client_error.__class__.__name__} -- [{client_error.__traceback__.tb_lineno}]: {client_error}"
                )
            return region, None
        except Exception as error:
            logger.error(
                f"{error.__class__.__name__} -- [{error.__traceback__.tb_lineno}]: {error}"
            )
            return region, None

    @staticmethod
    def verify_enabled_per_region(
        aws_security_hub_available_regions: list[str],
        session: Session,
        aws_account_id: str,
        aws_partition: str,
    ) -> dict[str, Session]:
        """
        Filters the given list of regions where AWS Security Hub is enabled and returns a dictionary containing the region and their boto3 client if the region and the Prowler integration is enabled.

        Args:
            aws_security_hub_available_regions (list[str]): List of AWS regions to check for Security Hub integration.

        Returns:
            dict: A dictionary containing enabled regions with SecurityHub clients.
        """
        enabled_regions = {}

        # Use ThreadPoolExecutor to check regions in parallel
        with ThreadPoolExecutor(
            max_workers=min(len(aws_security_hub_available_regions), 20)
        ) as executor:
            # Submit all region checks
            future_to_region = {
                executor.submit(
                    SecurityHub._check_region_security_hub,
                    region,
                    session,
                    aws_account_id,
                    aws_partition,
                ): region
                for region in aws_security_hub_available_regions
            }

            # Collect results as they complete
            for future in as_completed(future_to_region):
                try:
                    region, client = future.result()
                    if client is not None:
                        enabled_regions[region] = client
                except Exception as error:
                    logger.error(
                        f"Error checking region {future_to_region[future]}: {error.__class__.__name__} -- [{error.__traceback__.tb_lineno}]: {error}"
                    )

        return enabled_regions

    def batch_send_to_security_hub(
        self,
    ) -> int:
        """
        Sends the findings to AWS Security Hub in batches for each region and returns the count of successfully sent findings.

        Returns:
            int: Number of successfully sent findings to AWS Security Hub.
        """
        success_count = 0
        try:
            # Iterate findings by region
            for region, findings in self._findings_per_region.items():
                # Send findings to Security Hub
                logger.info(
                    f"Sending {len(findings)} findings to Security Hub in the region {region}"
                )

                # Convert findings to dict
                findings = [finding.dict(exclude_none=True) for finding in findings]
                success_count += self._send_findings_in_batches(
                    findings,
                    region,
                )

        except Exception as error:
            logger.error(
                f"{error.__class__.__name__} -- [{error.__traceback__.tb_lineno}]:{error} in region {region}"
            )
        return success_count

    def archive_previous_findings(self) -> int:
        """
        Checks previous findings in Security Hub to archive them.

        Returns:
            int: Number of successfully archived findings.
        """
        logger.info("Checking previous findings in Security Hub to archive them.")
        success_count = 0
        for region in self._findings_per_region.keys():
            try:
                current_findings = self._findings_per_region[region]
                # Get current findings IDs
                current_findings_ids = []
                for finding in current_findings:
                    current_findings_ids.append(finding.Id)
                # Get findings of that region
                findings_filter = {
                    "ProductName": [{"Value": "Prowler", "Comparison": "EQUALS"}],
                    "RecordState": [{"Value": "ACTIVE", "Comparison": "EQUALS"}],
                    "AwsAccountId": [
                        {"Value": self._aws_account_id, "Comparison": "EQUALS"}
                    ],
                    "Region": [{"Value": region, "Comparison": "EQUALS"}],
                }
                get_findings_paginator = self._enabled_regions[region].get_paginator(
                    "get_findings"
                )
                findings_to_archive = []
                for page in get_findings_paginator.paginate(
                    Filters=findings_filter, PaginationConfig={"PageSize": 100}
                ):
                    # Archive findings that have not appear in this execution
                    for finding in page["Findings"]:
                        if finding["Id"] not in current_findings_ids:
                            finding["RecordState"] = "ARCHIVED"
                            finding["UpdatedAt"] = timestamp_utc.strftime(
                                "%Y-%m-%dT%H:%M:%SZ"
                            )

                            findings_to_archive.append(finding)
                logger.info(f"Archiving {len(findings_to_archive)} findings.")

                # Send archive findings to SHub
                success_count += self._send_findings_in_batches(
                    findings_to_archive,
                    region,
                )
            except Exception as error:
                logger.error(
                    f"{error.__class__.__name__} -- [{error.__traceback__.tb_lineno}]:{error} in region {region}"
                )
        return success_count

    def _send_findings_in_batches(
        self, findings: list[AWSSecurityFindingFormat], region: str
    ) -> int:
        """
        Sends the given findings to AWS Security Hub in batches for a specific region and returns the count of successfully sent findings.

        Args:
            findings (list[AWSSecurityFindingFormat]): List of findings to send to AWS Security Hub.
            region (str): The AWS region where the findings will be sent.

        Returns:
            int: Number of successfully sent findings to AWS Security Hub.
        """
        success_count = 0
        try:
            list_chunked = [
                findings[i : i + SECURITY_HUB_MAX_BATCH]
                for i in range(0, len(findings), SECURITY_HUB_MAX_BATCH)
            ]
            for findings in list_chunked:
                batch_import = self._enabled_regions[region].batch_import_findings(
                    Findings=findings
                )
                if batch_import["FailedCount"] > 0:
                    failed_import = batch_import["FailedFindings"][0]
                    logger.error(
                        f"Failed to send findings to AWS Security Hub -- {failed_import['ErrorCode']} -- {failed_import['ErrorMessage']}"
                    )
                success_count += batch_import["SuccessCount"]
            return success_count
        except Exception as error:
            logger.error(
                f"{error.__class__.__name__} -- [{error.__traceback__.tb_lineno}]:{error} in region {region}"
            )
            return success_count

    @staticmethod
    def test_connection(
        aws_account_id: str,
        aws_partition: str = None,
        regions: set = None,
        raise_on_exception: bool = True,
        profile: str = None,
        aws_region: str = AWS_STS_GLOBAL_ENDPOINT_REGION,
        role_arn: str = None,
        role_session_name: str = ROLE_SESSION_NAME,
        session_duration: int = 3600,
        external_id: str = None,
        mfa_enabled: bool = False,
        aws_access_key_id: str = None,
        aws_secret_access_key: str = None,
        aws_session_token: Optional[str] = None,
    ) -> SecurityHubConnection:
        """
        Test the connection to AWS Security Hub by checking if Security Hub is enabled in the provided region
        and if the Prowler integration is active.

        Args:
            aws_account_id (str): AWS account ID to check for Prowler integration.
            aws_partition (str): AWS partition (e.g., aws, aws-cn, aws-us-gov).
            regions (set): Set of regions to check for Security Hub integration.
            raise_on_exception (bool): Whether to raise an exception if an error occurs.
            profile (str): AWS profile name to use for authentication.
            aws_region (str): AWS region to use for the session.
            role_arn (str): ARN of the IAM role to assume.
            role_session_name (str): Name for the role session.
            session_duration (int): Duration of the role session in seconds.
            external_id (str): External ID to use when assuming the role.
            mfa_enabled (bool): Whether MFA is enabled.
            aws_access_key_id (str): AWS access key ID.
            aws_secret_access_key (str): AWS secret access key.
            aws_session_token (str): AWS session token.

        Returns:
            SecurityHubConnection: An object that contains the result of the test connection operation.
                - is_connected (bool): Indicates whether the connection was successful.
                - error (Exception): An exception object if an error occurs during the connection test.
                - enabled_regions (set): Set of regions where Security Hub is enabled.
                - disabled_regions (set): Set of regions where Security Hub is disabled.
        """
        try:
            disabled_regions = set()
            enabled_regions = set()

            # Set up AWS session
            session = AwsProvider.setup_session(
                mfa=mfa_enabled,
                profile=profile,
                aws_access_key_id=aws_access_key_id,
                aws_secret_access_key=aws_secret_access_key,
                aws_session_token=aws_session_token,
            )
            if not aws_partition:
                aws_partition = AwsProvider.validate_credentials(
                    session, aws_region
                ).arn.partition

            # Handle role assumption if role_arn is provided
            if role_arn:
                session_duration = validate_session_duration(session_duration)
                role_session_name = validate_role_session_name(
                    role_session_name or ROLE_SESSION_NAME
                )
                role_arn = parse_iam_credentials_arn(role_arn)
                assumed_role_information = AWSAssumeRoleInfo(
                    role_arn=role_arn,
                    session_duration=session_duration,
                    external_id=external_id,
                    mfa_enabled=mfa_enabled,
                    role_session_name=role_session_name,
                )
                assumed_role_credentials = AwsProvider.assume_role(
                    session,
                    assumed_role_information,
                )
                session = Session(
                    aws_access_key_id=assumed_role_credentials.aws_access_key_id,
                    aws_secret_access_key=assumed_role_credentials.aws_secret_access_key,
                    aws_session_token=assumed_role_credentials.aws_session_token,
                    region_name=aws_region,
                    profile_name=profile,
                )

            all_regions = AwsProvider.get_available_aws_service_regions(
                service="securityhub", partition=aws_partition
            )

            enabled_regions = SecurityHub.verify_enabled_per_region(
                aws_security_hub_available_regions=all_regions,
                session=session,
                aws_account_id=aws_account_id,
                aws_partition=aws_partition,
            ).keys()
            disabled_regions = all_regions - enabled_regions
            if regions:
                if not any(region in enabled_regions for region in regions):
                    logger.warning(
                        f"Prowler integration is not enabled in regions: {regions - enabled_regions}."
                    )
                    invalid_region_error = SecurityHubInvalidRegionError(
                        message="Given regions have not Security Hub enabled."
                    )
                    if raise_on_exception:
                        raise invalid_region_error
                    return SecurityHubConnection(
                        is_connected=False,
                        error=invalid_region_error,
                        enabled_regions=enabled_regions,
                        disabled_regions=disabled_regions,
                    )
                else:
                    logger.info(
                        f"Prowler integration is enabled in regions: {', '.join(regions)}."
                    )
                    return SecurityHubConnection(
                        is_connected=True,
                        error=None,
                        enabled_regions=enabled_regions,
                        disabled_regions=disabled_regions,
                        partition=aws_partition,
                    )

            if len(enabled_regions) == 0:
                error_str = (
                    "No regions found with the Security Hub integration enabled."
                )
                logger.warning(error_str)
                no_enabled_regions_error = SecurityHubNoEnabledRegionsError(
                    message=error_str
                )
                if raise_on_exception:
                    raise no_enabled_regions_error
                return SecurityHubConnection(
                    is_connected=False,
                    error=no_enabled_regions_error,
                    enabled_regions=enabled_regions,
                    disabled_regions=disabled_regions,
                )
            else:
                logger.info(
                    f"Security Hub is enabled in the following regions: {', '.join(enabled_regions)}."
                )
                return SecurityHubConnection(
                    is_connected=True,
                    error=None,
                    enabled_regions=enabled_regions,
                    disabled_regions=disabled_regions,
                    partition=aws_partition,
                )

        except AWSSetUpSessionError as setup_session_error:
            logger.error(
                f"{setup_session_error.__class__.__name__}[{setup_session_error.__traceback__.tb_lineno}]: {setup_session_error}"
            )
            if raise_on_exception:
                raise setup_session_error
            return SecurityHubConnection(
                is_connected=False,
                error=setup_session_error,
                enabled_regions=set(),
                disabled_regions=set(),
            )

        except AWSArgumentTypeValidationError as validation_error:
            logger.error(
                f"{validation_error.__class__.__name__}[{validation_error.__traceback__.tb_lineno}]: {validation_error}"
            )
            if raise_on_exception:
                raise validation_error
            return SecurityHubConnection(
                is_connected=False,
                error=validation_error,
                enabled_regions=set(),
                disabled_regions=set(),
            )

        except AWSIAMRoleARNRegionNotEmtpyError as arn_region_not_empty_error:
            logger.error(
                f"{arn_region_not_empty_error.__class__.__name__}[{arn_region_not_empty_error.__traceback__.tb_lineno}]: {arn_region_not_empty_error}"
            )
            if raise_on_exception:
                raise arn_region_not_empty_error
            return SecurityHubConnection(
                is_connected=False,
                error=arn_region_not_empty_error,
                enabled_regions=set(),
                disabled_regions=set(),
            )

        except AWSIAMRoleARNPartitionEmptyError as arn_partition_empty_error:
            logger.error(
                f"{arn_partition_empty_error.__class__.__name__}[{arn_partition_empty_error.__traceback__.tb_lineno}]: {arn_partition_empty_error}"
            )
            if raise_on_exception:
                raise arn_partition_empty_error
            return SecurityHubConnection(
                is_connected=False,
                error=arn_partition_empty_error,
                enabled_regions=set(),
                disabled_regions=set(),
            )

        except AWSIAMRoleARNServiceNotIAMnorSTSError as arn_service_not_iam_sts_error:
            logger.error(
                f"{arn_service_not_iam_sts_error.__class__.__name__}[{arn_service_not_iam_sts_error.__traceback__.tb_lineno}]: {arn_service_not_iam_sts_error}"
            )
            if raise_on_exception:
                raise arn_service_not_iam_sts_error
            return SecurityHubConnection(
                is_connected=False,
                error=arn_service_not_iam_sts_error,
                enabled_regions=set(),
                disabled_regions=set(),
            )

        except AWSIAMRoleARNInvalidAccountIDError as arn_invalid_account_id_error:
            logger.error(
                f"{arn_invalid_account_id_error.__class__.__name__}[{arn_invalid_account_id_error.__traceback__.tb_lineno}]: {arn_invalid_account_id_error}"
            )
            if raise_on_exception:
                raise arn_invalid_account_id_error
            return SecurityHubConnection(
                is_connected=False,
                error=arn_invalid_account_id_error,
                enabled_regions=set(),
                disabled_regions=set(),
            )

        except AWSIAMRoleARNInvalidResourceTypeError as arn_invalid_resource_type_error:
            logger.error(
                f"{arn_invalid_resource_type_error.__class__.__name__}[{arn_invalid_resource_type_error.__traceback__.tb_lineno}]: {arn_invalid_resource_type_error}"
            )
            if raise_on_exception:
                raise arn_invalid_resource_type_error
            return SecurityHubConnection(
                is_connected=False,
                error=arn_invalid_resource_type_error,
                enabled_regions=set(),
                disabled_regions=set(),
            )

        except AWSIAMRoleARNEmptyResourceError as arn_empty_resource_error:
            logger.error(
                f"{arn_empty_resource_error.__class__.__name__}[{arn_empty_resource_error.__traceback__.tb_lineno}]: {arn_empty_resource_error}"
            )
            if raise_on_exception:
                raise arn_empty_resource_error
            return SecurityHubConnection(
                is_connected=False,
                error=arn_empty_resource_error,
                enabled_regions=set(),
                disabled_regions=set(),
            )

        except AWSAssumeRoleError as assume_role_error:
            logger.error(
                f"{assume_role_error.__class__.__name__}[{assume_role_error.__traceback__.tb_lineno}]: {assume_role_error}"
            )
            if raise_on_exception:
                raise assume_role_error
            return SecurityHubConnection(
                is_connected=False,
                error=assume_role_error,
                enabled_regions=set(),
                disabled_regions=set(),
            )

        except ProfileNotFound as profile_not_found_error:
            logger.error(
                f"AWSProfileNotFoundError[{profile_not_found_error.__traceback__.tb_lineno}]: {profile_not_found_error}"
            )
            if raise_on_exception:
                raise AWSProfileNotFoundError(
                    file=os.path.basename(__file__),
                    original_exception=profile_not_found_error,
                ) from profile_not_found_error
            return SecurityHubConnection(
                is_connected=False,
                error=profile_not_found_error,
                enabled_regions=set(),
                disabled_regions=set(),
            )

        except NoCredentialsError as no_credentials_error:
            logger.error(
                f"AWSNoCredentialsError[{no_credentials_error.__traceback__.tb_lineno}]: {no_credentials_error}"
            )
            if raise_on_exception:
                raise AWSNoCredentialsError(
                    file=os.path.basename(__file__),
                    original_exception=no_credentials_error,
                ) from no_credentials_error
            return SecurityHubConnection(
                is_connected=False,
                error=no_credentials_error,
                enabled_regions=set(),
                disabled_regions=set(),
            )

        except AWSAccessKeyIDInvalidError as access_key_id_invalid_error:
            logger.error(
                f"{access_key_id_invalid_error.__class__.__name__}[{access_key_id_invalid_error.__traceback__.tb_lineno}]: {access_key_id_invalid_error}"
            )
            if raise_on_exception:
                raise access_key_id_invalid_error
            return SecurityHubConnection(
                is_connected=False,
                error=access_key_id_invalid_error,
                enabled_regions=set(),
                disabled_regions=set(),
            )

        except AWSSecretAccessKeyInvalidError as secret_access_key_invalid_error:
            logger.error(
                f"{secret_access_key_invalid_error.__class__.__name__}[{secret_access_key_invalid_error.__traceback__.tb_lineno}]: {secret_access_key_invalid_error}"
            )
            if raise_on_exception:
                raise secret_access_key_invalid_error
            return SecurityHubConnection(
                is_connected=False,
                error=secret_access_key_invalid_error,
                enabled_regions=set(),
                disabled_regions=set(),
            )

        except AWSSessionTokenExpiredError as session_token_expired:
            logger.error(
                f"{session_token_expired.__class__.__name__}[{session_token_expired.__traceback__.tb_lineno}]: {session_token_expired}"
            )
            if raise_on_exception:
                raise session_token_expired
            return SecurityHubConnection(
                is_connected=False,
                error=session_token_expired,
                enabled_regions=set(),
                disabled_regions=set(),
            )

        except Exception as error:
            logger.error(
                f"{error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )
            if raise_on_exception:
                raise error
            return SecurityHubConnection(
                is_connected=False,
                error=error,
                enabled_regions=set(),
                disabled_regions=set(),
            )



from prowler.lib.check.models import Check, CheckReportAlibabaCloud
from prowler.providers.alibabacloud.services.sls.sls_client import sls_client


class sls_ram_role_changes_alert_enabled(Check):
    def execute(self) -> list[CheckReportAlibabaCloud]:
        findings = []
        found = False

        for alert in sls_client.alerts:
            # Check configuration for query
            # alert.configuration is a dict. configuration['queryList'] is a list of dicts with 'query'
            query_list = alert.configuration.get("queryList", [])
            if not query_list:
                continue

            for query_obj in query_list:
                query = query_obj.get("query", "")
                # Check for key terms
                # Query: ("event.serviceName": ResourceManager or "event.serviceName": Ram) ...
                if (
                    ("ResourceManager" in query or "Ram" in query)
                    and "CreatePolicy" in query
                    and "DeletePolicy" in query
                ):
                    found = True
                    report = CheckReportAlibabaCloud(
                        metadata=self.metadata(), resource=alert
                    )
                    report.status = "PASS"
                    report.status_extended = (
                        f"SLS Alert {alert.name} is configured for RAM Role changes."
                    )
                    report.resource_id = alert.name
                    report.resource_arn = alert.arn
                    report.region = alert.region
                    findings.append(report)
                    break  # Found one query in this alert

            if found:
                break

        if not found:
            report = CheckReportAlibabaCloud(
                metadata=self.metadata(), resource=sls_client.provider.identity
            )
            report.status = "FAIL"
            report.status_extended = "No SLS Alert configured for RAM Role changes."
            report.resource_id = sls_client.audited_account
            report.resource_arn = sls_client.provider.identity.identity_arn
            report.region = sls_client.region
            findings.append(report)

        return findings



import ssl
import urllib.error
import urllib.request

from prowler.lib.check.models import Check, Check_Report_AWS
from prowler.providers.aws.services.codepipeline.codepipeline_client import (
    codepipeline_client,
)


class codepipeline_project_repo_private(Check):
    """Checks if AWS CodePipeline source repositories are configured as private.

    This check verifies whether source repositories (GitHub or GitLab) connected to
    CodePipeline are publicly accessible. It attempts to access the repositories
    anonymously to determine their visibility status.

    Attributes:
        None
    """

    def execute(self) -> list:
        """Executes the repository privacy check for all CodePipeline sources.

        Iterates through all CodePipeline pipelines and checks if their source
        repositories (GitHub/GitLab) are publicly accessible by attempting anonymous
        access.

        Returns:
            list: List of Check_Report_AWS objects containing the findings for each
                pipeline's source repository.
        """
        findings = []

        for pipeline in codepipeline_client.pipelines.values():
            if (
                pipeline.source
                and pipeline.source.type == "CodeStarSourceConnection"
                and pipeline.source.repository_id
            ):
                report = Check_Report_AWS(self.metadata(), resource=pipeline)
                report.region = pipeline.region
                report.resource_id = pipeline.name
                report.resource_arn = pipeline.arn
                report.resource_tags = pipeline.tags

                # Try both GitHub and GitLab URLs
                github_url = f"https://github.com/{pipeline.source.repository_id}"
                gitlab_url = f"https://gitlab.com/{pipeline.source.repository_id}"

                is_public_github = self._is_public_repo(github_url)
                is_public_gitlab = self._is_public_repo(gitlab_url)

                if is_public_github:
                    report.status = "FAIL"
                    report.status_extended = f"CodePipeline {pipeline.name} source repository is public: {github_url}"
                elif is_public_gitlab:
                    report.status = "FAIL"
                    report.status_extended = f"CodePipeline {pipeline.name} source repository is public: {gitlab_url}"
                else:
                    report.status = "PASS"
                    report.status_extended = f"CodePipeline {pipeline.name} source repository {pipeline.source.repository_id} is private."

                findings.append(report)

        return findings

    def _is_public_repo(self, repo_url: str) -> bool:
        """Checks if a repository is publicly accessible.

        Attempts to access the repository URL anonymously to determine if it's
        public or private.

        Args:
            repo_url: String containing the repository URL to check.

        Returns:
            bool: True if the repository is public, False if private or inaccessible.

        Note:
            The method considers a repository private if:
            - The URL redirects to a sign-in page
            - The request fails with HTTP errors
            - The URL is not accessible
        """
        if repo_url.endswith(".git"):
            repo_url = repo_url[:-4]

        try:
            context = ssl._create_unverified_context()
            req = urllib.request.Request(repo_url, method="HEAD")
            response = urllib.request.urlopen(req, context=context)
            return not response.geturl().endswith("sign_in")
        except (urllib.error.HTTPError, urllib.error.URLError):
            return False



"""OCI Identity Service Module."""

from datetime import datetime
from typing import Optional

import oci
from pydantic import BaseModel

from prowler.lib.logger import logger
from prowler.providers.oraclecloud.lib.service.service import OCIService


class Identity(OCIService):
    """OCI Identity Service class to retrieve users, groups, policies, and authentication settings."""

    def __init__(self, provider):
        """
        Initialize the Identity service.

        Args:
            provider: The OCI provider instance
        """
        super().__init__("identity", provider)
        self.users = []
        self.groups = []
        self.policies = []
        self.dynamic_groups = []
        self.domains = []
        self.password_policy = None
        self.root_compartment_resources = []
        self.active_non_root_compartments = []
        self.__threading_call__(self.__list_users__)
        self.__threading_call__(self.__list_groups__)
        self.__threading_call__(self.__list_policies__)
        self.__threading_call__(self.__list_dynamic_groups__)
        self.__threading_call__(self.__list_domains__)
        self.__threading_call__(self.__list_domain_password_policies__)
        self.__get_password_policy__()
        self.__threading_call__(self.__search_root_compartment_resources__)
        self.__threading_call__(self.__search_active_non_root_compartments__)

    def __get_client__(self, region):
        """
        Get the Identity client for a region.

        Args:
            region: Region key

        Returns:
            Identity client instance
        """
        client_region = self.regional_clients.get(region)
        if client_region:
            return self._create_oci_client(oci.identity.IdentityClient)
        return None

    def __list_users__(self, regional_client):
        """
        List all IAM users in the tenancy.

        Args:
            regional_client: Regional OCI client
        """
        try:
            # Identity is a global service, use home region
            if regional_client.region not in self.provider.identity.region:
                return

            identity_client = self._create_oci_client(oci.identity.IdentityClient)

            logger.info("Identity - Listing Users...")

            for compartment in self.audited_compartments:
                try:
                    users = oci.pagination.list_call_get_all_results(
                        identity_client.list_users, compartment_id=compartment.id
                    ).data

                    for user in users:
                        if user.lifecycle_state != "DELETED":
                            # Get user API keys
                            api_keys = self.__list_user_api_keys__(
                                identity_client, user.id
                            )

                            # Get user auth tokens
                            auth_tokens = self.__list_user_auth_tokens__(
                                identity_client, user.id
                            )

                            # Get user customer secret keys
                            customer_secret_keys = (
                                self.__list_user_customer_secret_keys__(
                                    identity_client, user.id
                                )
                            )

                            # Get user database passwords
                            db_passwords = self.__list_user_db_passwords__(
                                identity_client, user.id
                            )

                            # Get user groups
                            user_groups = self.__get_user_groups__(
                                identity_client, user.id, compartment.id
                            )

                            # Check if user can use API keys
                            can_use_api_keys = (
                                user.capabilities.can_use_api_keys
                                if hasattr(user, "capabilities")
                                else True
                            )

                            # Check if console password is enabled
                            can_use_console_password = (
                                user.capabilities.can_use_console_password
                                if hasattr(user, "capabilities")
                                else False
                            )

                            # Check MFA status
                            is_mfa_activated = (
                                user.is_mfa_activated
                                if hasattr(user, "is_mfa_activated")
                                else False
                            )

                            self.users.append(
                                User(
                                    id=user.id,
                                    name=user.name,
                                    description=(
                                        user.description or ""
                                        if hasattr(user, "description")
                                        else ""
                                    ),
                                    email=(
                                        user.email or ""
                                        if hasattr(user, "email")
                                        else ""
                                    ),
                                    email_verified=(
                                        user.email_verified
                                        if hasattr(user, "email_verified")
                                        else False
                                    ),
                                    compartment_id=compartment.id,
                                    time_created=user.time_created,
                                    lifecycle_state=user.lifecycle_state,
                                    can_use_api_keys=can_use_api_keys,
                                    can_use_console_password=can_use_console_password,
                                    is_mfa_activated=is_mfa_activated,
                                    api_keys=api_keys,
                                    auth_tokens=auth_tokens,
                                    customer_secret_keys=customer_secret_keys,
                                    db_passwords=db_passwords,
                                    groups=user_groups,
                                    region=regional_client.region,
                                )
                            )
                except Exception as error:
                    logger.error(
                        f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                    )
                    continue
        except Exception as error:
            logger.error(
                f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )

    def __list_user_api_keys__(self, identity_client, user_id):
        """List API keys for a user."""
        try:
            api_keys = []
            api_keys_data = oci.pagination.list_call_get_all_results(
                identity_client.list_api_keys, user_id=user_id
            ).data

            for key in api_keys_data:
                api_keys.append(
                    ApiKey(
                        key_id=key.key_id,
                        fingerprint=key.fingerprint,
                        lifecycle_state=key.lifecycle_state,
                        time_created=key.time_created,
                        user_id=user_id,
                    )
                )
            return api_keys
        except Exception as error:
            logger.error(
                f"Identity - Error listing API keys for user {user_id}: {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )
            return []

    def __list_user_auth_tokens__(self, identity_client, user_id):
        """List auth tokens for a user."""
        try:
            auth_tokens = []
            auth_tokens_data = oci.pagination.list_call_get_all_results(
                identity_client.list_auth_tokens, user_id=user_id
            ).data

            for token in auth_tokens_data:
                auth_tokens.append(
                    AuthToken(
                        id=token.id,
                        description=(
                            token.description if hasattr(token, "description") else ""
                        ),
                        lifecycle_state=token.lifecycle_state,
                        time_created=token.time_created,
                        time_expires=(
                            token.time_expires
                            if hasattr(token, "time_expires")
                            else None
                        ),
                        user_id=user_id,
                    )
                )
            return auth_tokens
        except Exception as error:
            logger.error(
                f"Identity - Error listing auth tokens for user {user_id}: {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )
            return []

    def __list_user_customer_secret_keys__(self, identity_client, user_id):
        """List customer secret keys for a user."""
        try:
            customer_secret_keys = []
            keys_data = oci.pagination.list_call_get_all_results(
                identity_client.list_customer_secret_keys, user_id=user_id
            ).data

            for key in keys_data:
                customer_secret_keys.append(
                    CustomerSecretKey(
                        id=key.id,
                        display_name=(
                            key.display_name if hasattr(key, "display_name") else ""
                        ),
                        lifecycle_state=key.lifecycle_state,
                        time_created=key.time_created,
                        time_expires=(
                            key.time_expires if hasattr(key, "time_expires") else None
                        ),
                        user_id=user_id,
                    )
                )
            return customer_secret_keys
        except Exception as error:
            logger.error(
                f"Identity - Error listing customer secret keys for user {user_id}: {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )
            return []

    def __list_user_db_passwords__(self, identity_client, user_id):
        """List database passwords for a user."""
        try:
            db_passwords = []
            passwords_data = oci.pagination.list_call_get_all_results(
                identity_client.list_db_credentials, user_id=user_id
            ).data

            for password in passwords_data:
                db_passwords.append(
                    DbPassword(
                        id=password.id,
                        description=(
                            password.description
                            if hasattr(password, "description") and password.description
                            else None
                        ),
                        lifecycle_state=password.lifecycle_state,
                        time_created=password.time_created,
                        time_expires=(
                            password.time_expires
                            if hasattr(password, "time_expires")
                            else None
                        ),
                        user_id=user_id,
                    )
                )
            return db_passwords
        except Exception as error:
            logger.error(
                f"Identity - Error listing database passwords for user {user_id}: {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )
            return []

    def __get_user_groups__(self, identity_client, user_id, compartment_id):
        """Get groups for a user."""
        try:
            groups = []
            user_group_memberships = oci.pagination.list_call_get_all_results(
                identity_client.list_user_group_memberships,
                compartment_id=compartment_id,
                user_id=user_id,
            ).data

            for membership in user_group_memberships:
                if membership.lifecycle_state != "DELETED":
                    groups.append(membership.group_id)
            return groups
        except Exception as error:
            logger.error(
                f"Identity - Error getting groups for user {user_id}: {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )
            return []

    def __list_groups__(self, regional_client):
        """List all IAM groups."""
        try:
            if regional_client.region not in self.provider.identity.region:
                return

            identity_client = self._create_oci_client(oci.identity.IdentityClient)

            logger.info("Identity - Listing Groups...")

            for compartment in self.audited_compartments:
                try:
                    groups = oci.pagination.list_call_get_all_results(
                        identity_client.list_groups, compartment_id=compartment.id
                    ).data

                    for group in groups:
                        if group.lifecycle_state != "DELETED":
                            self.groups.append(
                                Group(
                                    id=group.id,
                                    name=group.name,
                                    description=(
                                        group.description
                                        if hasattr(group, "description")
                                        else ""
                                    ),
                                    compartment_id=compartment.id,
                                    time_created=group.time_created,
                                    lifecycle_state=group.lifecycle_state,
                                    region=regional_client.region,
                                )
                            )
                except Exception as error:
                    logger.error(
                        f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                    )
                    continue
        except Exception as error:
            logger.error(
                f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )

    def __list_policies__(self, regional_client):
        """List all IAM policies."""
        try:
            if regional_client.region not in self.provider.identity.region:
                return

            identity_client = self._create_oci_client(oci.identity.IdentityClient)

            logger.info("Identity - Listing Policies...")

            for compartment in self.audited_compartments:
                try:
                    policies = oci.pagination.list_call_get_all_results(
                        identity_client.list_policies, compartment_id=compartment.id
                    ).data

                    for policy in policies:
                        if policy.lifecycle_state != "DELETED":
                            self.policies.append(
                                Policy(
                                    id=policy.id,
                                    name=policy.name,
                                    description=(
                                        policy.description
                                        if hasattr(policy, "description")
                                        else ""
                                    ),
                                    compartment_id=compartment.id,
                                    statements=policy.statements,
                                    time_created=policy.time_created,
                                    lifecycle_state=policy.lifecycle_state,
                                    region=regional_client.region,
                                )
                            )
                except Exception as error:
                    logger.error(
                        f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                    )
                    continue
        except Exception as error:
            logger.error(
                f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )

    def __list_dynamic_groups__(self, regional_client):
        """List all dynamic groups in the tenancy."""
        try:
            # Dynamic groups are only in the home region
            if regional_client.region not in self.provider.identity.region:
                return

            identity_client = self._create_oci_client(oci.identity.IdentityClient)

            logger.info("Identity - Listing Dynamic Groups...")

            try:
                dynamic_groups = oci.pagination.list_call_get_all_results(
                    identity_client.list_dynamic_groups,
                    compartment_id=self.audited_tenancy,
                ).data

                for dynamic_group in dynamic_groups:
                    if dynamic_group.lifecycle_state != "DELETED":
                        self.dynamic_groups.append(
                            DynamicGroup(
                                id=dynamic_group.id,
                                name=dynamic_group.name,
                                description=(
                                    dynamic_group.description or ""
                                    if hasattr(dynamic_group, "description")
                                    else ""
                                ),
                                compartment_id=self.audited_tenancy,
                                matching_rule=(
                                    dynamic_group.matching_rule
                                    if hasattr(dynamic_group, "matching_rule")
                                    else ""
                                ),
                                time_created=dynamic_group.time_created,
                                lifecycle_state=dynamic_group.lifecycle_state,
                                region=regional_client.region,
                            )
                        )
            except Exception as error:
                logger.error(
                    f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                )
        except Exception as error:
            logger.error(
                f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )

    def __list_domains__(self, regional_client):
        """List all identity domains."""
        try:
            # Domains are only in the home region
            if regional_client.region not in self.provider.identity.region:
                return

            identity_client = self._create_oci_client(oci.identity.IdentityClient)

            logger.info("Identity - Listing Identity Domains...")

            try:
                # List all domains in the tenancy
                for compartment in self.audited_compartments:
                    domains = oci.pagination.list_call_get_all_results(
                        identity_client.list_domains,
                        compartment_id=compartment.id,
                        lifecycle_state="ACTIVE",
                    ).data

                    for domain in domains:
                        self.domains.append(
                            IdentityDomain(
                                id=domain.id,
                                display_name=domain.display_name,
                                description=domain.description or "",
                                url=domain.url,
                                home_region=domain.home_region,
                                compartment_id=compartment.id,
                                lifecycle_state=domain.lifecycle_state,
                                time_created=domain.time_created,
                                region=regional_client.region,
                                password_policies=[],
                            )
                        )

            except Exception as error:
                logger.error(
                    f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                )

        except Exception as error:
            logger.error(
                f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )

    def __list_domain_password_policies__(self, regional_client):
        """List password policies for all identity domains."""
        try:
            # Password policies are only in the home region
            if regional_client.region not in self.provider.identity.region:
                return

            logger.info("Identity - Listing Domain Password Policies...")

            for domain in self.domains:
                try:
                    # Create Identity Domains client for this domain
                    if self.session_signer:
                        domain_client = oci.identity_domains.IdentityDomainsClient(
                            config=self.session_config,
                            signer=self.session_signer,
                            service_endpoint=domain.url,
                        )
                    else:
                        domain_client = oci.identity_domains.IdentityDomainsClient(
                            config=self.session_config, service_endpoint=domain.url
                        )

                    # List password policies in the domain
                    policies_response = domain_client.list_password_policies()

                    for policy in policies_response.data.resources:
                        domain.password_policies.append(
                            DomainPasswordPolicy(
                                id=policy.id,
                                name=policy.name,
                                description=policy.description or "",
                                min_length=policy.min_length,
                                password_expires_after=policy.password_expires_after,
                                num_passwords_in_history=policy.num_passwords_in_history,
                                password_expire_warning=policy.password_expire_warning,
                                min_password_age=policy.min_password_age,
                            )
                        )

                except Exception as error:
                    logger.error(
                        f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                    )

        except Exception as error:
            logger.error(
                f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )

    def __get_password_policy__(self):
        """Get the password policy for the tenancy."""
        try:
            identity_client = self._create_oci_client(oci.identity.IdentityClient)

            logger.info("Identity - Getting Password Policy...")

            password_policy = identity_client.get_authentication_policy(
                compartment_id=self.audited_tenancy
            ).data.password_policy

            self.password_policy = PasswordPolicy(
                is_lowercase_characters_required=password_policy.is_lowercase_characters_required,
                is_uppercase_characters_required=password_policy.is_uppercase_characters_required,
                is_numeric_characters_required=password_policy.is_numeric_characters_required,
                is_special_characters_required=password_policy.is_special_characters_required,
                is_username_containment_allowed=password_policy.is_username_containment_allowed,
                minimum_password_length=password_policy.minimum_password_length,
            )
        except Exception as error:
            logger.error(
                f"Identity - Error getting password policy: {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )

    def __search_root_compartment_resources__(self, regional_client):
        """Search for resources in the root compartment using OCI Resource Search."""
        try:
            # Search is a global service, use home region
            if regional_client.region not in self.provider.identity.region:
                return

            logger.info("Identity - Searching for resources in root compartment...")

            # Create search client using the helper method for proper authentication
            search_client = self._create_oci_client(
                oci.resource_search.ResourceSearchClient
            )

            # Query to search for resources in root compartment
            # This covers VCN, instances, boot volumes, volumes, file systems, buckets,
            # autonomous databases, databases, and DB systems
            query_text = f"query VCN, instance, bootvolume, volume, filesystem, bucket, autonomousdatabase, database, dbsystem resources where compartmentId = '{self.audited_tenancy}'"

            # Execute structured search
            search_response = search_client.search_resources(
                search_details=oci.resource_search.models.StructuredSearchDetails(
                    type="Structured", query=query_text
                )
            )

            if search_response.data and search_response.data.items:
                for resource in search_response.data.items:
                    self.root_compartment_resources.append(
                        RootCompartmentResource(
                            display_name=resource.display_name or "",
                            identifier=resource.identifier,
                            resource_type=resource.resource_type,
                            compartment_id=resource.compartment_id,
                            availability_domain=getattr(
                                resource, "availability_domain", None
                            ),
                            lifecycle_state=getattr(resource, "lifecycle_state", None),
                            time_created=getattr(resource, "time_created", None),
                        )
                    )

        except Exception as error:
            logger.error(
                f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )

    def __search_active_non_root_compartments__(self, regional_client):
        """Search for active non-root compartments using OCI Resource Search."""
        try:
            # Search is a global service, use home region
            if regional_client.region not in self.provider.identity.region:
                return

            logger.info("Identity - Searching for active non-root compartments...")

            # Create search client using the helper method for proper authentication
            search_client = self._create_oci_client(
                oci.resource_search.ResourceSearchClient
            )

            # Query to search for active compartments in the tenancy (excluding root)
            query_text = f"query compartment resources where (compartmentId = '{self.audited_tenancy}' && lifecycleState = 'ACTIVE')"

            # Execute structured search
            search_response = search_client.search_resources(
                search_details=oci.resource_search.models.StructuredSearchDetails(
                    type="Structured", query=query_text
                )
            )

            if search_response.data and search_response.data.items:
                for compartment in search_response.data.items:
                    self.active_non_root_compartments.append(
                        ActiveCompartment(
                            display_name=compartment.display_name or "",
                            identifier=compartment.identifier,
                            compartment_id=compartment.compartment_id,
                            lifecycle_state=getattr(
                                compartment, "lifecycle_state", None
                            ),
                            time_created=getattr(compartment, "time_created", None),
                        )
                    )

        except Exception as error:
            logger.error(
                f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )


# Service Models
class ApiKey(BaseModel):
    """OCI API Key model."""

    key_id: str
    fingerprint: str
    lifecycle_state: str
    time_created: datetime
    user_id: str


class AuthToken(BaseModel):
    """OCI Auth Token model."""

    id: str
    description: str
    lifecycle_state: str
    time_created: datetime
    time_expires: Optional[datetime]
    user_id: str


class CustomerSecretKey(BaseModel):
    """OCI Customer Secret Key model."""

    id: str
    display_name: str
    lifecycle_state: str
    time_created: datetime
    time_expires: Optional[datetime]
    user_id: str


class DbPassword(BaseModel):
    """OCI Database Password model."""

    id: str
    description: Optional[str]
    lifecycle_state: str
    time_created: datetime
    time_expires: Optional[datetime]
    user_id: str


class User(BaseModel):
    """OCI IAM User model."""

    id: str
    name: str
    description: str
    email: str
    email_verified: bool
    compartment_id: str
    time_created: datetime
    lifecycle_state: str
    can_use_api_keys: bool
    can_use_console_password: bool
    is_mfa_activated: bool
    api_keys: list[ApiKey] = []
    auth_tokens: list[AuthToken] = []
    customer_secret_keys: list[CustomerSecretKey] = []
    db_passwords: list[DbPassword] = []
    groups: list[str] = []
    region: str


class Group(BaseModel):
    """OCI IAM Group model."""

    id: str
    name: str
    description: str
    compartment_id: str
    time_created: datetime
    lifecycle_state: str
    region: str


class Policy(BaseModel):
    """OCI IAM Policy model."""

    id: str
    name: str
    description: str
    compartment_id: str
    statements: list[str]
    time_created: datetime
    lifecycle_state: str
    region: str


class PasswordPolicy(BaseModel):
    """OCI Password Policy model."""

    is_lowercase_characters_required: bool
    is_uppercase_characters_required: bool
    is_numeric_characters_required: bool
    is_special_characters_required: bool
    is_username_containment_allowed: bool
    minimum_password_length: int


class AuthenticationPolicy(BaseModel):
    """OCI Authentication Policy model."""

    compartment_id: str
    password_policy: Optional[PasswordPolicy]


class DynamicGroup(BaseModel):
    """OCI Dynamic Group model."""

    id: str
    name: str
    description: str
    compartment_id: str
    matching_rule: str
    time_created: datetime
    lifecycle_state: str
    region: str


class DomainPasswordPolicy(BaseModel):
    """OCI Identity Domain Password Policy model."""

    id: str
    name: str
    description: str
    min_length: Optional[int]
    password_expires_after: Optional[int]
    num_passwords_in_history: Optional[int]
    password_expire_warning: Optional[int]
    min_password_age: Optional[int]


class IdentityDomain(BaseModel):
    """OCI Identity Domain model."""

    id: str
    display_name: str
    description: str
    url: str
    home_region: str
    compartment_id: str
    lifecycle_state: str
    time_created: datetime
    region: str
    password_policies: list[DomainPasswordPolicy]


class RootCompartmentResource(BaseModel):
    """OCI Resource found in root compartment via search."""

    display_name: str
    identifier: str
    resource_type: str
    compartment_id: str
    availability_domain: Optional[str]
    lifecycle_state: Optional[str]
    time_created: Optional[datetime]


class ActiveCompartment(BaseModel):
    """OCI Active non-root compartment found via search."""

    display_name: str
    identifier: str
    compartment_id: str
    lifecycle_state: Optional[str]
    time_created: Optional[datetime]



from prowler.lib.logger import logger
from prowler.lib.mutelist.mutelist import Mutelist
from prowler.lib.outputs.utils import unroll_dict, unroll_tags


class OCIMutelist(Mutelist):
    """
    OCIMutelist class manages the mutelist functionality for OCI provider.

    This class extends the base Mutelist class to provide OCI-specific functionality
    for muting findings based on tenancy, check, region, resource, and tags.
    """

    def __init__(
        self,
        mutelist_content: dict = {},
        mutelist_path: str = None,
        tenancy_id: str = "",
    ) -> "OCIMutelist":
        """
        Initialize the OCIMutelist.

        Args:
            mutelist_content (dict): The mutelist content as a dictionary.
            mutelist_path (str): The path to the mutelist file.
            tenancy_id (str): The OCI tenancy ID.
        """
        self._mutelist = mutelist_content
        self._mutelist_file_path = mutelist_path
        self._tenancy_id = tenancy_id

        if mutelist_path:
            # Load mutelist from local file
            self.get_mutelist_file_from_local_file(mutelist_path)

        if self._mutelist:
            self._mutelist = self.validate_mutelist(self._mutelist)

    def is_finding_muted(
        self,
        finding,
        tenancy_id: str,
    ) -> bool:
        """
        Check if a finding is muted based on the mutelist.

        Args:
            finding: The finding object to check (should have check_metadata, region, resource_id, resource_tags).
            tenancy_id (str): The OCI tenancy ID.

        Returns:
            bool: True if the finding is muted, False otherwise.
        """
        try:
            check_id = finding.check_metadata.CheckID
            region = finding.region if hasattr(finding, "region") else ""
            resource_id = finding.resource_id if hasattr(finding, "resource_id") else ""
            resource_tags = {}

            # Handle resource tags
            if hasattr(finding, "resource_tags") and finding.resource_tags:
                resource_tags = unroll_dict(unroll_tags(finding.resource_tags))

            return self.is_muted(
                tenancy_id,
                check_id,
                region,
                resource_id,
                resource_tags,
            )
        except Exception as error:
            logger.error(
                f"{error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )
            return False

    def is_muted(
        self,
        tenancy_id: str,
        check_id: str,
        region: str,
        resource_id: str,
        resource_tags: dict,
    ) -> bool:
        """
        Check if a specific combination is muted.

        Args:
            tenancy_id (str): The OCI tenancy ID.
            check_id (str): The check ID.
            region (str): The OCI region.
            resource_id (str): The resource ID (OCID).
            resource_tags (dict): The resource tags.

        Returns:
            bool: True if muted, False otherwise.
        """
        try:
            if not self._mutelist:
                return False

            # Check if mutelist has Accounts/Tenancies section
            tenancies = self._mutelist.get("Accounts", {})
            if not tenancies:
                # Try with "Tenancies" key for OCI-specific mutelist
                tenancies = self._mutelist.get("Tenancies", {})

            # Check for wildcard or specific tenancy
            tenancy_mutelist = tenancies.get("*", {})
            if tenancy_id in tenancies:
                # Merge with specific tenancy rules
                specific_tenancy = tenancies.get(tenancy_id, {})
                tenancy_mutelist = {**tenancy_mutelist, **specific_tenancy}

            if not tenancy_mutelist:
                return False

            # Get checks for this tenancy
            checks = tenancy_mutelist.get("Checks", {})

            # Check for wildcard or specific check
            check_mutelist = checks.get("*", {})
            if check_id in checks:
                specific_check = checks.get(check_id, {})
                check_mutelist = {**check_mutelist, **specific_check}

            if not check_mutelist:
                return False

            # Check regions
            regions = check_mutelist.get("Regions", [])
            if regions and "*" not in regions and region not in regions:
                return False

            # Check resources
            resources = check_mutelist.get("Resources", [])
            if resources:
                if "*" not in resources and resource_id not in resources:
                    return False

            # Check tags
            tags = check_mutelist.get("Tags", [])
            if tags and resource_tags:
                # Check if any tag matches
                tag_match = False
                for tag_filter in tags:
                    # Tag filter format: "key=value" or "key=*"
                    if "=" in tag_filter:
                        key, value = tag_filter.split("=", 1)
                        if key in resource_tags:
                            if value == "*" or resource_tags[key] == value:
                                tag_match = True
                                break

                if not tag_match:
                    return False

            # Check exceptions (resources that should NOT be muted)
            exceptions = check_mutelist.get("Exceptions", {})
            if exceptions:
                exception_resources = exceptions.get("Resources", [])
                if resource_id in exception_resources:
                    return False

                exception_regions = exceptions.get("Regions", [])
                if region in exception_regions:
                    return False

            # If we passed all checks, the finding is muted
            return True

        except Exception as error:
            logger.error(
                f"{error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )
            return False



from typing import Optional

from botocore.exceptions import ClientError
from pydantic import BaseModel

from prowler.lib.logger import logger
from prowler.providers.aws.lib.service.service import AWSService


class CodePipeline(AWSService):
    """AWS CodePipeline service class for managing pipeline resources.

    This class handles interactions with AWS CodePipeline service, including
    listing pipelines and retrieving their states. It manages pipeline resources
    and their associated metadata.

    Attributes:
        pipelines: Dictionary mapping pipeline ARNs to Pipeline objects.
    """

    def __init__(self, provider):
        """Initializes the CodePipeline service class.

        Args:
            provider: AWS provider instance for making API calls.
        """
        super().__init__(__class__.__name__, provider)
        self.pipelines = {}
        self.__threading_call__(self._list_pipelines)
        if self.pipelines:
            self.__threading_call__(self._get_pipeline_state, self.pipelines.values())
            self.__threading_call__(
                self._list_tags_for_resource, self.pipelines.values()
            )

    def _list_pipelines(self, regional_client):
        """Lists all CodePipeline pipelines in the specified region.

        Retrieves all pipelines using pagination and creates Pipeline objects
        for each pipeline found.

        Args:
            regional_client: AWS regional client for CodePipeline service.

        Raises:
            ClientError: If there is an AWS API error.
        """
        logger.info("CodePipeline - Listing pipelines...")
        try:
            list_pipelines_paginator = regional_client.get_paginator("list_pipelines")
            for page in list_pipelines_paginator.paginate():
                for pipeline in page["pipelines"]:
                    pipeline_arn = f"arn:{self.audited_partition}:codepipeline:{regional_client.region}:{self.audited_account}:{pipeline['name']}"
                    if self.pipelines is None:
                        self.pipelines = {}
                    self.pipelines[pipeline_arn] = Pipeline(
                        name=pipeline["name"],
                        arn=pipeline_arn,
                        region=regional_client.region,
                    )
        except ClientError as error:
            if error.response["Error"]["Code"] == "AccessDenied":
                logger.error(
                    f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                )
                if not self.pipelines:
                    self.pipelines = None
            else:
                logger.error(
                    f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                )
        except Exception as error:
            logger.error(
                f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )

    def _get_pipeline_state(self, pipeline):
        """Retrieves the current state of a pipeline.

        Gets detailed information about a pipeline including its source configuration.

        Args:
            pipeline: Pipeline object to retrieve state for.

        Raises:
            ClientError: If there is an AWS API error.
        """
        logger.info("CodePipeline - Getting pipeline state...")
        try:
            regional_client = self.regional_clients[pipeline.region]
            pipeline_info = regional_client.get_pipeline(name=pipeline.name)
            source_info = pipeline_info["pipeline"]["stages"][0]["actions"][0]
            repository_id = source_info["configuration"].get("FullRepositoryId", "")
            pipeline.source = Source(
                type=source_info["actionTypeId"]["provider"],
                repository_id=repository_id,
                configuration=source_info["configuration"],
            )
        except ClientError as error:
            logger.error(
                f"{pipeline.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )
        except Exception as error:
            logger.error(
                f"{pipeline.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )

    def _list_tags_for_resource(self, resource):
        """Lists tags for a given resource.

        Args:
            resource: Resource object to retrieve tags for.
        """
        logger.info("CodePipeline - Listing Tags...")
        try:
            tags_response = self.regional_clients[
                resource.region
            ].list_tags_for_resource(resourceArn=resource.arn)
            resource.tags = tags_response.get("tags", [])
        except ClientError as error:
            if error.response["Error"]["Code"] == "ResourceNotFoundException":
                logger.warning(
                    f"{resource.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                )
            else:
                logger.error(
                    f"{resource.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                )
        except Exception as error:
            logger.error(
                f"{resource.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )


class Source(BaseModel):
    """Model representing a pipeline source configuration.

    Attributes:
        type: The type of source provider.
        location: The location/path of the source repository.
        configuration: Optional dictionary containing additional source configuration.
    """

    type: str
    repository_id: str
    configuration: Optional[dict]


class Pipeline(BaseModel):
    """Model representing an AWS CodePipeline pipeline.

    Attributes:
        name: The name of the pipeline.
        arn: The ARN (Amazon Resource Name) of the pipeline.
        region: The AWS region where the pipeline exists.
        source: Optional Source object containing source configuration.
        tags: Optional list of pipeline tags.
    """

    name: str
    arn: str
    region: str
    source: Optional[Source] = None
    tags: Optional[list] = []



from typing import List

from prowler.lib.check.models import Check, CheckReportGithub
from prowler.providers.github.services.organization.organization_client import (
    organization_client,
)


class organization_default_repository_permission_strict(Check):
    """Check if an organization's base repository permission is set to a strict level.

    PASS: base permission is "read" or "none"
    FAIL: base permission is "write" or "admin" (or any other non-strict value)
    """

    def execute(self) -> List[CheckReportGithub]:
        findings = []
        for org in organization_client.organizations.values():
            base_perm = getattr(org, "base_permission", None)
            if base_perm is None:
                # Unknown / no permission to read → skip producing a finding
                continue

            p = str(base_perm).lower()
            report = CheckReportGithub(metadata=self.metadata(), resource=org)

            if p in ("read", "none"):
                report.status = "PASS"
                report.status_extended = f"Organization {org.name} base repository permission is '{p}', which is strict."
            else:
                report.status = "FAIL"
                report.status_extended = f"Organization {org.name} base repository permission is '{p}', which is not strict."

            findings.append(report)

        return findings



from pydantic.v1 import BaseModel

import prowler.providers.gcp.config as config
from prowler.lib.logger import logger
from prowler.providers.gcp.gcp_provider import GcpProvider
from prowler.providers.gcp.lib.service.service import GCPService
from prowler.providers.gcp.services.cloudresourcemanager.cloudresourcemanager_client import (
    cloudresourcemanager_client,
)


class AccessContextManager(GCPService):
    def __init__(self, provider: GcpProvider):
        super().__init__("accesscontextmanager", provider, api_version="v1")
        self.service_perimeters = []
        self._get_service_perimeters()

    def _get_service_perimeters(self):
        for org in cloudresourcemanager_client.organizations:
            try:
                access_policies = []
                try:
                    request = self.client.accessPolicies().list(
                        parent=f"organizations/{org.id}"
                    )
                    while request is not None:
                        response = request.execute(
                            num_retries=config.DEFAULT_RETRY_ATTEMPTS
                        )
                        access_policies.extend(response.get("accessPolicies", []))

                        request = self.client.accessPolicies().list_next(
                            previous_request=request, previous_response=response
                        )
                except Exception as error:
                    logger.error(
                        f"{self.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                    )
                    continue

                for policy in access_policies:
                    try:
                        request = (
                            self.client.accessPolicies()
                            .servicePerimeters()
                            .list(parent=policy["name"])
                        )
                        while request is not None:
                            response = request.execute(
                                num_retries=config.DEFAULT_RETRY_ATTEMPTS
                            )

                            for perimeter in response.get("servicePerimeters", []):
                                status = perimeter.get("status", {})
                                spec = perimeter.get("spec", {})

                                perimeter_config = status if status else spec

                                resources = perimeter_config.get("resources", [])
                                restricted_services = perimeter_config.get(
                                    "restrictedServices", []
                                )

                                self.service_perimeters.append(
                                    ServicePerimeter(
                                        name=perimeter["name"],
                                        title=perimeter.get("title", ""),
                                        perimeter_type=perimeter.get(
                                            "perimeterType", ""
                                        ),
                                        resources=resources,
                                        restricted_services=restricted_services,
                                        policy_name=policy["name"],
                                    )
                                )

                            request = (
                                self.client.accessPolicies()
                                .servicePerimeters()
                                .list_next(
                                    previous_request=request, previous_response=response
                                )
                            )
                    except Exception as error:
                        logger.error(
                            f"{self.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                        )

            except Exception as error:
                logger.error(
                    f"{self.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                )


class ServicePerimeter(BaseModel):
    name: str
    title: str
    perimeter_type: str
    resources: list[str]
    restricted_services: list[str]
    policy_name: str



"""Check Ensure Oracle Analytics Cloud (OAC) access is restricted to allowed sources or deployed within a Virtual Cloud Network."""

from prowler.lib.check.models import Check, Check_Report_OCI
from prowler.providers.oraclecloud.services.analytics.analytics_client import (
    analytics_client,
)


class analytics_instance_access_restricted(Check):
    """Check Ensure Oracle Analytics Cloud (OAC) access is restricted to allowed sources or deployed within a Virtual Cloud Network."""

    def execute(self) -> Check_Report_OCI:
        """Execute the analytics_instance_access_restricted check."""
        findings = []

        for instance in analytics_client.analytics_instances:
            report = Check_Report_OCI(
                metadata=self.metadata(),
                resource=instance,
                region=instance.region,
                resource_name=instance.name,
                resource_id=instance.id,
                compartment_id=instance.compartment_id,
            )

            # Check if instance has PUBLIC network endpoint type
            if (
                instance.network_endpoint_type
                and instance.network_endpoint_type.upper() == "PUBLIC"
            ):
                # Check if whitelisted IPs are configured
                if not instance.whitelisted_ips:
                    report.status = "FAIL"
                    report.status_extended = f"Analytics instance {instance.name} has public access with no whitelisted IPs configured."
                # Check if 0.0.0.0/0 is in whitelisted IPs
                elif "0.0.0.0/0" in instance.whitelisted_ips:
                    report.status = "FAIL"
                    report.status_extended = f"Analytics instance {instance.name} has public access with unrestricted IP range (0.0.0.0/0)."
                else:
                    report.status = "PASS"
                    report.status_extended = f"Analytics instance {instance.name} has public access with restricted whitelisted IPs."
            else:
                report.status = "PASS"
                report.status_extended = f"Analytics instance {instance.name} is deployed within a VCN or has restricted access."

            findings.append(report)

        return findings



"""Check Ensure permissions on all resources are given only to the tenancy administrator group."""

from prowler.lib.check.models import Check, Check_Report_OCI
from prowler.providers.oraclecloud.services.identity.identity_client import (
    identity_client,
)


class identity_tenancy_admin_permissions_limited(Check):
    """Check Ensure permissions on all resources are given only to the tenancy administrator group."""

    def execute(self) -> Check_Report_OCI:
        """Execute the identity_tenancy_admin_permissions_limited check.

        Ensure permissions on all resources are given only to the tenancy administrator group.
        This check verifies that only the 'Tenant Admin Policy' grants 'manage all-resources in tenancy' permissions.
        Other policies should not have such broad permissions.
        """
        findings = []

        # Check for policies that grant "manage all-resources in tenancy"
        for policy in identity_client.policies:
            # Skip non-active policies
            if policy.lifecycle_state != "ACTIVE":
                continue

            region = policy.region if hasattr(policy, "region") else "global"

            has_violation = False
            for statement in policy.statements:
                statement_upper = statement.upper()

                # Check for "allow group ... to manage all-resources in tenancy"
                # This should only be in "Tenant Admin Policy"
                if (
                    "ALLOW GROUP" in statement_upper
                    and "TO MANAGE ALL-RESOURCES IN TENANCY" in statement_upper
                ):
                    # If this is not the Tenant Admin Policy, it's a violation
                    if policy.name.upper() != "TENANT ADMIN POLICY":
                        has_violation = True
                    break

            report = Check_Report_OCI(
                metadata=self.metadata(),
                resource=policy,
                region=region,
                resource_name=policy.name,
                resource_id=policy.id,
                compartment_id=policy.compartment_id,
            )

            if has_violation:
                report.status = "FAIL"
                report.status_extended = f"Policy '{policy.name}' grants 'manage all-resources in tenancy' permissions to groups other than the Administrators group. Only the tenancy administrator group should have such broad permissions."
            else:
                report.status = "PASS"
                report.status_extended = f"Policy '{policy.name}' does not grant overly broad tenancy-wide permissions to non-administrator groups."

            findings.append(report)

        # If no policies found, that's a PASS (no violations)
        if not findings:
            region = (
                identity_client.audited_regions[0].key
                if identity_client.audited_regions
                else "global"
            )
            report = Check_Report_OCI(
                metadata=self.metadata(),
                resource={},
                region=region,
                resource_name="Tenancy",
                resource_id=identity_client.audited_tenancy,
                compartment_id=identity_client.audited_tenancy,
            )
            report.status = "PASS"
            report.status_extended = "No active policies found granting overly broad tenancy-wide permissions to non-administrator groups."
            findings.append(report)

        return findings



from prowler.lib.check.models import Check, CheckReportAlibabaCloud
from prowler.providers.alibabacloud.services.ram.ram_client import ram_client


class ram_password_policy_lowercase(Check):
    """Check if RAM password policy requires at least one lowercase letter."""

    def execute(self) -> list[CheckReportAlibabaCloud]:
        findings = []

        if ram_client.password_policy:
            report = CheckReportAlibabaCloud(
                metadata=self.metadata(), resource=ram_client.password_policy
            )
            report.region = ram_client.region
            report.resource_id = f"{ram_client.audited_account}-password-policy"
            report.resource_arn = (
                f"acs:ram::{ram_client.audited_account}:password-policy"
            )

            if ram_client.password_policy.require_lowercase_characters:
                report.status = "PASS"
                report.status_extended = (
                    "RAM password policy requires at least one lowercase letter."
                )
            else:
                report.status = "FAIL"
                report.status_extended = "RAM password policy does not require at least one lowercase letter."

            findings.append(report)

        return findings



import time
from threading import current_thread
from typing import Any, Dict, List, Optional

import requests
from requests.auth import HTTPDigestAuth

from prowler.lib.logger import logger
from prowler.providers.mongodbatlas.exceptions.exceptions import (
    MongoDBAtlasAPIError,
    MongoDBAtlasRateLimitError,
)


class MongoDBAtlasService:
    """Base class for MongoDB Atlas services"""

    def __init__(self, service_name: str, provider):
        self.service_name = service_name
        self.provider = provider
        self.session = provider.session
        self.base_url = provider.session.base_url
        self.audit_config = provider.audit_config
        self.auth = HTTPDigestAuth(
            provider.session.public_key, provider.session.private_key
        )
        self.headers = {
            "Accept": "application/vnd.atlas.2025-01-01+json",
            "Content-Type": "application/json",
        }

    def _make_request(
        self,
        method: str,
        endpoint: str,
        params: Optional[Dict] = None,
        data: Optional[Dict] = None,
        max_retries: int = 3,
        retry_delay: int = 1,
    ) -> Dict[str, Any]:
        """
        Make HTTP request to MongoDB Atlas API with retry logic

        Args:
            method: HTTP method (GET, POST, PUT, DELETE)
            endpoint: API endpoint (without base URL)
            params: Query parameters
            data: Request body data
            max_retries: Maximum number of retries
            retry_delay: Delay between retries in seconds

        Returns:
            dict: Response JSON data

        Raises:
            MongoDBAtlasAPIError: If the API request fails
            MongoDBAtlasRateLimitError: If rate limit is exceeded
        """
        url = f"{self.base_url}/{endpoint.lstrip('/')}"

        for attempt in range(max_retries + 1):
            try:
                response = requests.request(
                    method=method,
                    url=url,
                    auth=self.auth,
                    headers=self.headers,
                    params=params,
                    json=data,
                    timeout=30,
                )

                if response.status_code == 429:
                    if attempt < max_retries:
                        logger.warning(
                            f"Rate limit exceeded for {url}, retrying in {retry_delay} seconds..."
                        )
                        time.sleep(retry_delay)
                        retry_delay *= 2
                        continue
                    else:
                        raise MongoDBAtlasRateLimitError(
                            message=f"Rate limit exceeded for {url} after {max_retries} retries"
                        )

                response.raise_for_status()
                return response.json()

            except requests.exceptions.RequestException as e:
                if attempt < max_retries:
                    logger.warning(
                        f"Request failed for {url}, retrying in {retry_delay} seconds: {str(e)}"
                    )
                    time.sleep(retry_delay)
                    retry_delay *= 2
                    continue
                else:
                    logger.error(
                        f"Request failed for {url} after {max_retries} retries: {str(e)}"
                    )
                    raise MongoDBAtlasAPIError(
                        original_exception=e,
                        message=f"Failed to make request to {url}: {str(e)}",
                    )

    def _paginate_request(
        self,
        endpoint: str,
        params: Optional[Dict] = None,
        page_size: int = 100,
        max_pages: Optional[int] = None,
    ) -> List[Dict[str, Any]]:
        """
        Make paginated requests to MongoDB Atlas API

        Args:
            endpoint: API endpoint
            params: Query parameters
            page_size: Number of items per page
            max_pages: Maximum number of pages to fetch

        Returns:
            list: List of all items from all pages
        """
        if params is None:
            params = {}

        params.update({"pageNum": 1, "itemsPerPage": page_size})

        all_items = []
        page_num = 1

        while True:
            params["pageNum"] = page_num

            try:
                response = self._make_request("GET", endpoint, params=params)

                if "results" in response:
                    items = response["results"]
                    all_items.extend(items)

                    total_count = response.get("totalCount", 0)

                    if len(items) < page_size or len(all_items) >= total_count:
                        break

                    if max_pages and page_num >= max_pages:
                        logger.warning(
                            f"Reached maximum pages limit ({max_pages}) for {endpoint}"
                        )
                        break

                    page_num += 1
                else:
                    break

            except Exception as e:
                logger.error(
                    f"Error during pagination for {endpoint} at page {page_num}: {str(e)}"
                )
                break

        logger.info(
            f"Retrieved {len(all_items)} items from {endpoint} across {page_num} pages"
        )

        return all_items

    def _get_thread_info(self) -> str:
        """Get thread information for logging"""
        return f"[{current_thread().name}]"



from prowler.lib.check.models import Check, CheckReportAlibabaCloud
from prowler.providers.alibabacloud.services.securitycenter.securitycenter_client import (
    securitycenter_client,
)


class securitycenter_all_assets_agent_installed(Check):
    """Check if all assets are installed with security agent."""

    def execute(self) -> list[CheckReportAlibabaCloud]:
        findings = []

        uninstalled_machines = securitycenter_client.uninstalled_machines

        if not uninstalled_machines:
            # All assets have the agent installed
            report = CheckReportAlibabaCloud(metadata=self.metadata(), resource={})
            report.region = securitycenter_client.region
            report.resource_id = securitycenter_client.audited_account
            report.resource_arn = (
                f"acs:sas::{securitycenter_client.audited_account}:security-center"
            )
            report.status = "PASS"
            report.status_extended = "All assets have Security Center agent installed."
            findings.append(report)
        else:
            # Report each uninstalled machine
            for machine in uninstalled_machines:
                report = CheckReportAlibabaCloud(
                    metadata=self.metadata(), resource=machine
                )
                report.region = machine.region
                report.resource_id = machine.instance_id
                report.resource_arn = (
                    f"acs:ecs:{machine.region}:{securitycenter_client.audited_account}:instance/{machine.instance_id}"
                    if machine.instance_id.startswith("i-")
                    or "ecs" in machine.instance_id.lower()
                    else f"acs:sas:{machine.region}:{securitycenter_client.audited_account}:machine/{machine.instance_id}"
                )
                report.status = "FAIL"
                report.status_extended = (
                    f"Asset {machine.instance_name if machine.instance_name else machine.instance_id} "
                    f"({machine.instance_id}) does not have Security Center agent installed. "
                    f"Region: {machine.region}, OS: {machine.os if machine.os else 'Unknown'}."
                )
                findings.append(report)

        return findings



from datetime import datetime, timedelta, timezone

from prowler.lib.check.models import Check, CheckReportAlibabaCloud
from prowler.providers.alibabacloud.services.ram.ram_client import ram_client


class ram_rotate_access_key_90_days(Check):
    """Check if access keys are rotated every 90 days or less."""

    def execute(self) -> list[CheckReportAlibabaCloud]:
        findings = []
        # Use UTC timezone-aware datetime for consistent comparison
        now = datetime.now(timezone.utc)
        ninety_days_ago = now - timedelta(days=90)

        for user in ram_client.users:
            if user.access_keys:
                for access_key in user.access_keys:
                    # Only check active access keys
                    if access_key.status == "Active":
                        report = CheckReportAlibabaCloud(
                            metadata=self.metadata(), resource=user
                        )
                        report.region = ram_client.region
                        report.resource_id = access_key.access_key_id
                        report.resource_arn = f"acs:ram::{ram_client.audited_account}:user/{user.name}/accesskey/{access_key.access_key_id}"

                        if access_key.create_date:
                            # Ensure create_date is timezone-aware for comparison
                            create_date = access_key.create_date
                            if create_date.tzinfo is None:
                                # If naive, assume UTC
                                create_date = create_date.replace(tzinfo=timezone.utc)

                            if create_date < ninety_days_ago:
                                report.status = "FAIL"
                                days_old = (now - create_date).days
                                report.status_extended = (
                                    f"Access key {access_key.access_key_id} for user {user.name} "
                                    f"has not been rotated in {days_old} days (more than 90 days)."
                                )
                            else:
                                report.status = "PASS"
                                days_old = (now - create_date).days
                                report.status_extended = (
                                    f"Access key {access_key.access_key_id} for user {user.name} "
                                    f"was created {days_old} days ago (within 90 days)."
                                )
                        else:
                            report.status = "PASS"
                            report.status_extended = (
                                f"Access key {access_key.access_key_id} for user {user.name} "
                                f"creation date is not available."
                            )

                        findings.append(report)

        return findings



from prowler.providers.alibabacloud.services.securitycenter.securitycenter_service import (
    SecurityCenter,
)
from prowler.providers.common.provider import Provider

securitycenter_client = SecurityCenter(Provider.get_global_provider())



from prowler.lib.check.models import Check, Check_Report_Azure
from prowler.providers.azure.services.postgresql.postgresql_client import (
    postgresql_client,
)


class postgresql_flexible_server_entra_id_authentication_enabled(Check):
    def execute(self) -> Check_Report_Azure:
        findings = []
        for (
            subscription,
            flexible_servers,
        ) in postgresql_client.flexible_servers.items():
            for server in flexible_servers:
                report = Check_Report_Azure(metadata=self.metadata(), resource=server)
                report.subscription = subscription
                # Default to FAIL
                report.status = "FAIL"

                # Check if Entra ID authentication is enabled
                # Note: active_directory_auth is already normalized to uppercase in service layer
                if (
                    not server.active_directory_auth
                    or server.active_directory_auth != "ENABLED"
                ):
                    report.status_extended = f"Flexible Postgresql server {server.name} from subscription {subscription} has Microsoft Entra ID authentication disabled"
                else:
                    # Authentication is enabled, now check for admins
                    admin_count = (
                        len(server.entra_id_admins) if server.entra_id_admins else 0
                    )

                    if admin_count == 0:
                        report.status_extended = f"Flexible Postgresql server {server.name} from subscription {subscription} has Microsoft Entra ID authentication enabled but no Entra ID administrators configured"
                    else:
                        report.status = "PASS"
                        admin_text = (
                            "administrator" if admin_count == 1 else "administrators"
                        )
                        report.status_extended = f"Flexible Postgresql server {server.name} from subscription {subscription} has Microsoft Entra ID authentication enabled with {admin_count} {admin_text} configured"
                findings.append(report)

        return findings



"""OCI Network Service Module."""

from datetime import datetime
from typing import Optional

import oci
from pydantic import BaseModel

from prowler.lib.logger import logger
from prowler.providers.oraclecloud.lib.service.service import OCIService


class Network(OCIService):
    """OCI Network Service class to retrieve VCNs, security lists, and network security groups."""

    def __init__(self, provider):
        """
        Initialize the Network service.

        Args:
            provider: The OCI provider instance
        """
        super().__init__("network", provider)
        self.vcns = []
        self.security_lists = []
        self.network_security_groups = []
        self.subnets = []
        self.__threading_call_by_region_and_compartment__(self.__list_vcns__)
        self.__threading_call_by_region_and_compartment__(self.__list_security_lists__)
        self.__threading_call_by_region_and_compartment__(
            self.__list_network_security_groups__
        )
        self.__threading_call_by_region_and_compartment__(self.__list_subnets__)

    def __get_client__(self, region):
        """
        Get the VirtualNetwork client for a region.

        Args:
            region: Region key

        Returns:
            VirtualNetwork client instance
        """
        return self._create_oci_client(
            oci.core.VirtualNetworkClient, config_overrides={"region": region}
        )

    def __list_vcns__(self, region, compartment):
        """
        List all VCNs in a compartment and region.

        Args:
            region: OCIRegion object
            compartment: Compartment object
        """
        try:
            region_key = region.key if hasattr(region, "key") else str(region)
            vcn_client = self.__get_client__(region_key)

            logger.info(
                f"Network - Listing VCNs in {region_key} - {compartment.name}..."
            )

            vcns_data = oci.pagination.list_call_get_all_results(
                vcn_client.list_vcns, compartment_id=compartment.id
            ).data

            for vcn in vcns_data:
                if vcn.lifecycle_state != "TERMINATED":
                    # Get default security list
                    default_security_list_id = vcn.default_security_list_id

                    self.vcns.append(
                        VCN(
                            id=vcn.id,
                            display_name=(
                                vcn.display_name if hasattr(vcn, "display_name") else ""
                            ),
                            compartment_id=compartment.id,
                            cidr_blocks=(
                                vcn.cidr_blocks if hasattr(vcn, "cidr_blocks") else []
                            ),
                            lifecycle_state=vcn.lifecycle_state,
                            default_security_list_id=default_security_list_id,
                            region=region_key,
                            time_created=vcn.time_created,
                        )
                    )
        except Exception as error:
            logger.error(
                f"{region_key} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )

    def __list_security_lists__(self, region, compartment):
        """
        List all security lists in a compartment and region.

        Args:
            region: OCIRegion object
            compartment: Compartment object
        """
        try:
            region_key = region.key if hasattr(region, "key") else str(region)
            vcn_client = self.__get_client__(region_key)

            logger.info(
                f"Network - Listing Security Lists in {region_key} - {compartment.name}..."
            )

            security_lists_data = oci.pagination.list_call_get_all_results(
                vcn_client.list_security_lists, compartment_id=compartment.id
            ).data

            for sec_list in security_lists_data:
                if sec_list.lifecycle_state != "TERMINATED":
                    # Check if this is a default security list
                    is_default = False
                    vcn_id = sec_list.vcn_id
                    for vcn in self.vcns:
                        if (
                            vcn.id == vcn_id
                            and vcn.default_security_list_id == sec_list.id
                        ):
                            is_default = True
                            break

                    # Convert OCI SDK objects to dict for JSON serialization
                    ingress_rules = [
                        oci.util.to_dict(rule)
                        for rule in (sec_list.ingress_security_rules or [])
                    ]
                    egress_rules = [
                        oci.util.to_dict(rule)
                        for rule in (sec_list.egress_security_rules or [])
                    ]

                    self.security_lists.append(
                        SecurityList(
                            id=sec_list.id,
                            display_name=(
                                sec_list.display_name
                                if hasattr(sec_list, "display_name")
                                else ""
                            ),
                            compartment_id=compartment.id,
                            vcn_id=sec_list.vcn_id,
                            ingress_security_rules=ingress_rules,
                            egress_security_rules=egress_rules,
                            lifecycle_state=sec_list.lifecycle_state,
                            is_default=is_default,
                            region=region_key,
                            time_created=sec_list.time_created,
                        )
                    )
        except Exception as error:
            logger.error(
                f"{region_key} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )

    def __list_network_security_groups__(self, region, compartment):
        """
        List all network security groups in a compartment and region.

        Args:
            region: OCIRegion object
            compartment: Compartment object
        """
        try:
            region_key = region.key if hasattr(region, "key") else str(region)
            vcn_client = self.__get_client__(region_key)

            logger.info(
                f"Network - Listing Network Security Groups in {region_key} - {compartment.name}..."
            )

            nsgs_data = oci.pagination.list_call_get_all_results(
                vcn_client.list_network_security_groups, compartment_id=compartment.id
            ).data

            for nsg in nsgs_data:
                if nsg.lifecycle_state != "TERMINATED":
                    # Get NSG rules
                    try:
                        nsg_rules_data = oci.pagination.list_call_get_all_results(
                            vcn_client.list_network_security_group_security_rules,
                            network_security_group_id=nsg.id,
                        ).data
                        # Convert OCI SDK objects to dict for JSON serialization
                        nsg_rules = [oci.util.to_dict(rule) for rule in nsg_rules_data]
                    except Exception:
                        nsg_rules = []

                    self.network_security_groups.append(
                        NetworkSecurityGroup(
                            id=nsg.id,
                            display_name=(
                                nsg.display_name if hasattr(nsg, "display_name") else ""
                            ),
                            compartment_id=compartment.id,
                            vcn_id=nsg.vcn_id,
                            security_rules=nsg_rules,
                            lifecycle_state=nsg.lifecycle_state,
                            region=region_key,
                            time_created=nsg.time_created,
                        )
                    )
        except Exception as error:
            logger.error(
                f"{region_key} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )

    def __list_subnets__(self, region, compartment):
        """
        List all subnets in a compartment and region.

        Args:
            region: OCIRegion object
            compartment: Compartment object
        """
        try:
            region_key = region.key if hasattr(region, "key") else str(region)
            vcn_client = self.__get_client__(region_key)

            logger.info(
                f"Network - Listing Subnets in {region_key} - {compartment.name}..."
            )

            subnets_data = oci.pagination.list_call_get_all_results(
                vcn_client.list_subnets, compartment_id=compartment.id
            ).data

            for subnet in subnets_data:
                if subnet.lifecycle_state != "TERMINATED":
                    self.subnets.append(
                        Subnet(
                            id=subnet.id,
                            display_name=(
                                subnet.display_name
                                if hasattr(subnet, "display_name")
                                else ""
                            ),
                            compartment_id=compartment.id,
                            vcn_id=subnet.vcn_id,
                            cidr_block=(
                                subnet.cidr_block
                                if hasattr(subnet, "cidr_block")
                                else ""
                            ),
                            lifecycle_state=subnet.lifecycle_state,
                            region=region_key,
                            time_created=subnet.time_created,
                        )
                    )
        except Exception as error:
            logger.error(
                f"{region_key} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )


# Service Models
class VCN(BaseModel):
    """OCI VCN model."""

    id: str
    display_name: str
    compartment_id: str
    cidr_blocks: list[str]
    lifecycle_state: str
    default_security_list_id: Optional[str]
    region: str
    time_created: datetime


class SecurityList(BaseModel):
    """OCI Security List model."""

    id: str
    display_name: str
    compartment_id: str
    vcn_id: str
    ingress_security_rules: list
    egress_security_rules: list
    lifecycle_state: str
    is_default: bool = False
    region: str
    time_created: datetime

    class Config:
        arbitrary_types_allowed = True
        json_encoders = {object: lambda v: str(v)}


class NetworkSecurityGroup(BaseModel):
    """OCI Network Security Group model."""

    id: str
    display_name: str
    compartment_id: str
    vcn_id: str
    security_rules: list
    lifecycle_state: str
    region: str
    time_created: datetime

    class Config:
        arbitrary_types_allowed = True
        json_encoders = {object: lambda v: str(v)}


class Subnet(BaseModel):
    """OCI Subnet model."""

    id: str
    display_name: str
    compartment_id: str
    vcn_id: str
    cidr_block: str
    lifecycle_state: str
    region: str
    time_created: datetime



"""OCI Compute Service Module."""

from typing import Optional

import oci
from pydantic import BaseModel

from prowler.lib.logger import logger
from prowler.providers.oraclecloud.lib.service.service import OCIService


class Compute(OCIService):
    """OCI Compute Service class to retrieve compute instances."""

    def __init__(self, provider):
        """
        Initialize the Compute service.

        Args:
            provider: The OCI provider instance
        """
        super().__init__("compute", provider)
        self.instances = []
        self.__threading_call__(self.__list_instances__)

    def __get_client__(self, region):
        """
        Get the Compute client for a region.

        Args:
            region: Region key

        Returns:
            Compute client instance
        """
        client_region = self.regional_clients.get(region)
        if client_region:
            return self._create_oci_client(oci.core.ComputeClient)
        return None

    def __list_instances__(self, regional_client):
        """
        List all compute instances in the compartments.

        Args:
            regional_client: Regional OCI client
        """
        try:
            compute_client = self.__get_client__(regional_client.region)
            if not compute_client:
                return

            logger.info(f"Compute - Listing Instances in {regional_client.region}...")

            for compartment in self.audited_compartments:
                try:
                    instances = oci.pagination.list_call_get_all_results(
                        compute_client.list_instances, compartment_id=compartment.id
                    ).data

                    for instance in instances:
                        if instance.lifecycle_state not in [
                            "TERMINATED",
                            "TERMINATING",
                        ]:
                            # Get instance metadata options
                            metadata_options = (
                                instance.instance_options.are_legacy_imds_endpoints_disabled
                                if hasattr(instance, "instance_options")
                                and hasattr(
                                    instance.instance_options,
                                    "are_legacy_imds_endpoints_disabled",
                                )
                                else None
                            )

                            # Get secure boot status
                            is_secure_boot_enabled = (
                                instance.platform_config.is_secure_boot_enabled
                                if hasattr(instance, "platform_config")
                                and hasattr(
                                    instance.platform_config, "is_secure_boot_enabled"
                                )
                                else False
                            )

                            # Get in-transit encryption status from launch options
                            is_pv_encryption_in_transit_enabled = (
                                instance.launch_options.is_pv_encryption_in_transit_enabled
                                if hasattr(instance, "launch_options")
                                and hasattr(
                                    instance.launch_options,
                                    "is_pv_encryption_in_transit_enabled",
                                )
                                else None
                            )

                            self.instances.append(
                                Instance(
                                    id=instance.id,
                                    name=(
                                        instance.display_name
                                        if hasattr(instance, "display_name")
                                        else instance.id
                                    ),
                                    compartment_id=compartment.id,
                                    region=regional_client.region,
                                    lifecycle_state=instance.lifecycle_state,
                                    are_legacy_imds_endpoints_disabled=metadata_options,
                                    is_secure_boot_enabled=is_secure_boot_enabled,
                                    is_pv_encryption_in_transit_enabled=is_pv_encryption_in_transit_enabled,
                                )
                            )
                except Exception as error:
                    logger.error(
                        f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
                    )
                    continue
        except Exception as error:
            logger.error(
                f"{regional_client.region} -- {error.__class__.__name__}[{error.__traceback__.tb_lineno}]: {error}"
            )


# Service Models
class Instance(BaseModel):
    """OCI Compute Instance model."""

    id: str
    name: str
    compartment_id: str
    region: str
    lifecycle_state: str
    are_legacy_imds_endpoints_disabled: Optional[bool] = None
    is_secure_boot_enabled: bool = False
    is_pv_encryption_in_transit_enabled: Optional[bool] = None



from prowler.lib.check.models import Check, Check_Report_Azure
from prowler.providers.azure.services.app.app_client import app_client


class app_function_access_keys_configured(Check):
    def execute(self):
        findings = []

        for (
            subscription_name,
            functions,
        ) in app_client.functions.items():
            for function in functions.values():
                if function.function_keys is not None:
                    report = Check_Report_Azure(
                        metadata=self.metadata(), resource=function
                    )
                    report.subscription = subscription_name
                    report.status = "FAIL"
                    report.status_extended = f"Function {function.name} does not have function keys configured."

                    if len(function.function_keys) > 0:
                        report.status = "PASS"
                        report.status_extended = (
                            f"Function {function.name} has function keys configured."
                        )

                    findings.append(report)

        return findings



from typing import List

from prowler.lib.check.models import Check, CheckReportGithub
from prowler.providers.github.services.repository.repository_client import (
    repository_client,
)


class repository_immutable_releases_enabled(Check):
    """Ensure immutable releases are enabled for GitHub repositories.

    Immutable releases prevent post-publication tampering of binaries and release assets.
    """

    def execute(self) -> List[CheckReportGithub]:
        """Run the immutable releases verification for each discovered repository.

        Returns:
            List[CheckReportGithub]: Collection of check reports describing the immutable releases status.
        """
        findings: List[CheckReportGithub] = []
        for repo in repository_client.repositories.values():
            if repo.immutable_releases_enabled is None:
                continue

            report = CheckReportGithub(metadata=self.metadata(), resource=repo)

            if repo.immutable_releases_enabled:
                report.status = "PASS"
                report.status_extended = (
                    f"Repository {repo.name} has immutable releases enabled."
                )
            else:
                report.status = "FAIL"
                report.status_extended = (
                    f"Repository {repo.name} does not have immutable releases enabled."
                )

            findings.append(report)

        return findings



from prowler.lib.check.models import Check, CheckReportAlibabaCloud
from prowler.providers.alibabacloud.services.actiontrail.actiontrail_client import (
    actiontrail_client,
)


class actiontrail_multi_region_enabled(Check):
    """Check if ActionTrail is configured to export copies of all log entries."""

    def execute(self) -> list[CheckReportAlibabaCloud]:
        findings = []

        # Check if there's at least one multi-region trail that is enabled
        multi_region_trails = []
        for trail in actiontrail_client.trails.values():
            if trail.trail_region == "All" and trail.status == "Enable":
                multi_region_trails.append(trail)

        # Create a single report for the overall check
        report = CheckReportAlibabaCloud(metadata=self.metadata(), resource={})
        report.region = actiontrail_client.region
        report.resource_id = actiontrail_client.audited_account
        report.resource_arn = (
            f"acs:actiontrail::{actiontrail_client.audited_account}:trail"
        )

        if multi_region_trails:
            # At least one multi-region trail is enabled
            trail_names = [trail.name for trail in multi_region_trails]
            report.status = "PASS"
            report.status_extended = (
                f"ActionTrail is configured with {len(multi_region_trails)} multi-region trail(s) "
                f"that are enabled: {', '.join(trail_names)}. "
                "These trails export copies of all log entries across all regions."
            )
        else:
            # Check if there are any trails at all
            if actiontrail_client.trails:
                # There are trails but none are multi-region or enabled
                enabled_trails = [
                    t
                    for t in actiontrail_client.trails.values()
                    if t.status == "Enable"
                ]
                multi_region_trails_disabled = [
                    t
                    for t in actiontrail_client.trails.values()
                    if t.trail_region == "All" and t.status != "Enable"
                ]

                if enabled_trails and not multi_region_trails_disabled:
                    report.status = "FAIL"
                    report.status_extended = (
                        f"ActionTrail has {len(enabled_trails)} enabled trail(s), but none are configured "
                        "for multi-region logging (TrailRegion is not set to 'All'). "
                        "Multi-region trails are required to capture events from all regions."
                    )
                elif multi_region_trails_disabled:
                    trail_names = [t.name for t in multi_region_trails_disabled]
                    report.status = "FAIL"
                    report.status_extended = (
                        f"ActionTrail has multi-region trail(s) but they are disabled: {', '.join(trail_names)}. "
                        "Enable the multi-region trail(s) to export copies of all log entries."
                    )
                else:
                    report.status = "FAIL"
                    report.status_extended = (
                        "ActionTrail has trails configured, but none are enabled or configured for multi-region logging. "
                        "At least one trail with TrailRegion set to 'All' and Status set to 'Enable' is required."
                    )
            else:
                # No trails configured at all
                report.status = "FAIL"
                report.status_extended = (
                    "ActionTrail is not configured. No trails exist. "
                    "Create at least one multi-region trail (TrailRegion='All') and enable it "
                    "to export copies of all log entries across all regions."
                )

        findings.append(report)
        return findings
